# -*- coding: utf-8 -*-

# Copyright (c) 2006-2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPMainCore.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# The MainCoreCallbacks class, that's the thing that controls the events except
# activity stuff, is using ocempgui.
# The menu items are ocempgui buttons.
# Sprites are only used in the activities.
# The activity menu and the bottom menu bar are controlled by MainCoreGui

import sys
import gc
import atexit
import os
import pygame
import threading
import textwrap
import types
import ConfigParser
import subprocess
import datetime
from pygame.constants import *
import time

#create logger, logger was configured in SPLogging
import logging
module_logger = logging.getLogger("childsplay.SPMainCore")

import utils
###### for debugging memoryleaks ###########
#from guppy import hpy                     #
#h = hpy()                                 #
############################################
from SPSpriteUtils import SPInit, MySprite, set_big_mouse_cursor, set_no_mouse_cursor, SPGroup
from SPGoodies import SPGoodies

# This is a kludge to execute the toplevel code in SPConstants as it is already
# imported in SPlogging before the loggers are set.
import SPConstants
reload(SPConstants)
from SPConstants import *

try:
    import psyco #@UnresolvedImport
    #psyco.log(PSYCOPATH, 'w')
    psyco.profile()
except:
    pass
else:
    module_logger.debug("We got psyco, we should go faster, but report any weird crashes.")

import SPMenu
import SPDataManager as SPDataManager
import Version
import SPVideoPlayer

from SPWidgets import Init, Dialog, Label, MenuBar, VolumeAdjust, ExeCounter, \
                    Graph, TransImgButton
#from SPVirtualkeyboard import VirtualKeyboard
try:
    from sqlalchemy import exceptions as sqlae
except ImportError:
    from sqlalchemy import exc as sqlae

# Used to cleanup stuff when the Python vm ends
def cleanup():
    module_logger.info("cleanup called.")
    module_logger.info("flushing dbase to disk")
    # flush dbase?
    # remove any locks that been set
    utils._remove_lock()
    module_logger.info("logger stops.")
    # add cleanup stuff in here or register it like the Activity class does
#    index_off_php = os.path.join(WWWDIR, 'index_off.php')
#    index_php = os.path.join(WWWDIR, 'index.php')
#    if os.path.exists(index_php):
#        os.remove(index_php)
#    shutil.copy(index_off_php, index_php)
    
atexit.register(cleanup)

class MainEscapeKeyException(Exception):
    """Raised when the user decides to quit the maincore.
    This is catched by schoolsplay.py to check if we quit or go back to the login"""
    pass
class EscapeKeyException(Exception):
    """ This is raised from the activity_loop when the user hits escape.
    We basically using this exception as a signal"""
    pass
class LevelsEndException(Exception):
    """ This is raised from activity_loop when there are no more levels in the game."""
    pass
class GDMEscapeKeyException(Exception):
    """ This is raised from the activity_loop when the user hits escape.
    We basically using this exception as a signal"""
    pass
    
class MainCoreGui:
    def __init__(self, resolution=(800, 600), dbmaker=None, options=None, \
                 fullscr=None, mainscr=None, error=False,session_id=None):
        """The main SP core.
        The idea is that we have a menubar and a activity area.
        The menu bar is the place for "our" gui widgets.
        
        The activity doesn't draw in the menu bar and we don't come
        into the activity area.
        We call from the eventloop first some local checks like pressing escape etc.
        Then we call the actives_group in which our activity sprites are
        held.
        Finally we call the the eventloop of the activity.
        """
        self.logger = logging.getLogger("childsplay.SPMainCore.MainCoreGui")
        self.logger.debug("Start")
        self.myspritesactivated = None
        self.menubuttons_backup = []
        self.volume_level = 50
        self.levelcount = 1
        self.DTmode = False
        self.DTactivity = None
        self.DTlevelscounter = 1
        self.DTlevelcount = 3
        self.DTact_is_finished = None
        self.COPmode = False
        self.cmd_options = options
        self.theme_rc = self.cmd_options.theme_rc
        self.logger.debug("Found menu rc options %s" % self.theme_rc)
        self.theme = self.cmd_options.theme
        language = self.cmd_options.lang
        dbase = os.path.join(HOMEDIR, self.theme, 'quizcontent.db')
        self.foreign_observers = []
        
        # we setup the stuff we need for the stats table data.
        # we collect evrything in a hash and when we done we put it into a
        # dbase table.
        self.statshash = {'session':session_id}
        # get our base options from a rc file.
        self.core_rc_path = os.path.join(ACTIVITYDATADIR, 'SPData', 'base', 'themes','core.rc')
        self.logger.debug("Parsing core rc file %s" % self.core_rc_path)
        d = {}
        config = ConfigParser.ConfigParser()
        config.read(self.core_rc_path)
        for k, v in dict(config.items(self.theme)).items():
            d[k] = v
        d['theme'] = self.theme
        self.core_rc = d
        self.logger.debug("found core rc options %s" % d)
                
        if self.cmd_options.no_sound:
            self.logger.info("Disabling sound support as requested by user")
            pygame.mixer.quit()
                    
        # set mouse cursor ?
        # TODO: use different cursors with an cmdline option
        if self.cmd_options.bigcursor:
            cursorfile = os.path.join(ACTIVITYDATADIR, 'SPData', 'base', 'icons', 'cursor_1.xbm')
            maskfile = os.path.join(ACTIVITYDATADIR, 'SPData', 'base', 'icons', 'cursor_1-mask.xbm')
            set_big_mouse_cursor(cursorfile, maskfile)
        if self.cmd_options.nocursor:
            set_no_mouse_cursor()
        self.resolution = resolution
        self.vtkb = None # will become the virtual keyboard object
        
        ## Set icon for the window manager ## 
        ICONPATH = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', self.theme)
        DEFAULTICONPATH = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', 'childsplay')
        # TODO:change icon files to the ones we would use
        if sys.platform == 'darwin':
            file = 'logo_cp_180x180.png'
        elif sys.platform == 'win32':
            file = 'logo_cp_64x64.png'
        else:
            file = 'logo_cp_32x32.xpm'
        if not os.path.exists(os.path.join(ICONPATH, file)):
            icon_path = os.path.join(DEFAULTICONPATH, file)
        else:
            icon_path = os.path.join(ICONPATH, file)
        try:
            surface = pygame.image.load(icon_path)
        except pygame.error:
            self.logger.info('Could not load image "%s"\n %s' % (file, pygame.get_error()))
        else:
            surface.set_colorkey(surface.get_at((0, 0)), RLEACCEL)
            pygame.display.set_icon(surface)
        
        # Thread lock, must be used by activities to be able to block everything.
        self.lock = threading.Lock() 
        
        # setup main screen
        if self.cmd_options.fullscreen:
            if mainscr:
                self.screen = mainscr
            else:
                self.screen = pygame.display.set_mode(self.resolution, FULLSCREEN)
        else:
            if mainscr:
                self.screen = mainscr
            else:
                self.screen = pygame.display.set_mode(self.resolution)
        if self.theme_rc['menubar_position'] == 'bottom':
            y = 500
            blitpos_y = 0
            self.menu_rect = pygame.Rect(0, 500, 800, 100)
        else:# we only have two options: top or bottom
            y = 0
            blitpos_y = 100
            self.menu_rect = pygame.Rect(0, 0, 800, 100)                
        captxt = _("Childsplay_sp - educational activities, using theme:%s") % self.theme
        if self.cmd_options.theme != 'childsplay':
            captxt = captxt.replace('Childsplay_sp', self.cmd_options.theme)
        pygame.display.set_caption(captxt.encode('utf-8'))
        
        self.backgroundimage = utils.load_image(os.path.join(ICONPATH, self.theme_rc['background']))
        self.screen.blit(self.backgroundimage, (0, 0))
        img = utils.load_image(os.path.join(ICONPATH, self.theme_rc['menubartop']))
                
        self.backgroundimage.blit(img, (0, y))
        self.screen.blit(self.backgroundimage, (0, 0))
        self.backgr = self.screen.convert()# a real copy
        # init SpriteUtils, mandatory when using SpriteUtils
        self.actives_group = SPInit(self.screen, self.backgr, self.cmd_options.__dict__)
        self.actives_group.set_onematch(True)
        # Must be called before using SPWidgets
        Init(self.theme)
        
        # setup SPGoodies which we pass to the activities
        # from here on we set various attributes for the spgoodies instance 
        self.screenclip = None
        self.spgoodies = SPGoodies(self, self.screen, \
            self.get_virtkb, \
            language, \
            self.lock, \
            self.activity_pre_level_end, \
            self.activity_level_end, \
            self.activity_game_end, \
            self.activity_info_dialog, \
            self.display_execounter, \
            self.set_framerate, \
            self.enable_dice, \
            self.set_dice_minimal_level, \
            self.theme_rc, \
            self.cmd_options.theme,\
            self.disable_menubuttons, self.enable_menubuttons,\
            self.disable_level_indicator,self.enable_level_indicator, \
            self.disable_exit_button, self.enable_exit_button, \
            self.disable_score_button, self.enable_score_button, \
            self.hide_level_indicator, self.show_level_indicator, \
            self.get_orm,self.register_foreign_observer)
        # special goodies for the menu
        self.spgoodies._menu_activity_userchoice = self._menu_activity_userchoice
        self.spgoodies._cmd_options = self.cmd_options
        self.spgoodies.stats_hash = self.statshash  
        # and one for the quizengine
        self.spgoodies._unmute_quiz_voice = True
             
        if error:
            # there was an error in sp and the core is restarted
            self.activity_info_dialog(_("An error occurred and BraintrainerPlus was restarted"))

        # we get a datamanager which will start the SPgdm/BTP login screen, if available,
        # and does some username and table checking.
        try:
            self.dm = SPDataManager.DataManager(self.spgoodies, dbmaker)
        except sqlae.SQLAlchemyError:
            self.logger.exception("Error while handling the dbase, try removing the existing dbase")
        except utils.MyError, info:
            self.logger.error("%s" % info)
            raise utils.SPError
        # get locale setting from dbase and reset any locales already set to the 
        # commandline language option
        if not self.cmd_options.lang:
            language = self.dm._get_language()
            self.spgoodies.localesetting = language
        else:
            language = (language, None)
            self.spgoodies.localesetting = language
        # get session id, needed for the stats hash
        # get user data
        user_id = self.dm.get_user_id()
        name = self.dm.get_username()
        surf = utils.char2surf(name.replace('_', ' '), fcol=GREY ,fsize=12, bold=True)
        self.backgroundimage.blit(surf,((800 - surf.get_width()) / 2 ,70))
        orm, session = self.dm.get_orm('users', 'user')
        row = session.query(orm).filter_by(user_id = user_id).first()
        self.statshash['user_id'] = user_id
        self.statshash['datetime'] = datetime.datetime.now()# we override it when something is loaded
        # we now have user data so we check to see if we should set some custom stuff 
        # Set volume level
        if not row:
            self.logger.warning("Not found userdata for user id:%s" % user_id)
            self.volume_level = 75
        else:
            self.volume_level = int(row.audio)
            self.logger.debug("found volume setting %s for user id %s" % (self.volume_level, user_id))
        try:
            subprocess.Popen("amixer set Master %s" % self.volume_level + "%",shell=True)
            cmd=subprocess.Popen("amixer get Master",shell=True, \
                                 stdout=subprocess.PIPE, \
                                 stderr=subprocess.PIPE)
            output = cmd.communicate()[0]
        except Exception, info:
            module_logger.warning("program 'amixer' not found, unable to set volume levels: %s" % info)
            self.volume_level = 75
        else:
            for line in output.split('\n'):
                if "%]" in line:
                    self.volume_level = int(line.split("%]")[0].split("[")[1])
        # Check to see if today it's the user birthday
        if row and row.birthdate:
            name = "%s %s" % (row.title, row.last_name)
            if not os.path.exists(os.path.join(TEMPDIR, 'birthday', name)):
                date = row.birthdate
                today = datetime.date.today()
                if today.day == date.day and today.month == date.month:
                    age = int(today.year) - int(date.year)
                    try:
                        import birthday
                    except ImportError:
                        self.logger.warning("no birthday module found.")
                    else:
                        self.logger.debug("Start birthday show")
                        bd = birthday.birthday(self.screen, self.backgr,name, age)
                        bd.run()
                        f = open(os.path.join(TEMPDIR, 'birthday', name), 'w')
                        f.write('1')
                        f.close()
        if row and row.levelup_dlg != 'true':
            self.no_levelup_dlg = True
        else:
            self.no_levelup_dlg = False
        if self.dm.are_we_cop():
            self.COPmode = True        
            self.statshash['is_cop'] = True
        # restore window title after the datamanager replaced it
        pygame.display.set_caption(captxt.encode('utf-8'))
        if self.COPmode:
            theme_dir = os.path.join('controlpanel_lgpl','lib', 'SPData')
            xmlname = self.dm.are_we_cop()
            self.logger.info("Using different xml for our menu, were in COP mode")
            # we must rebuild the screen before we construct the menu
            self.backgroundimage = utils.load_image\
                        (os.path.join(theme_dir, 'controlpanel_main.png'))
            self.screen.blit(self.backgroundimage, (0, 0))
            img = utils.load_image(os.path.join(theme_dir, 'controlpanel_top.png'))

            self.backgroundimage.blit(img, (0, y))
            self.screen.blit(self.backgroundimage, (0, 0))
            self.backgr = self.screen.convert()# a real copy
            # init SpriteUtils, mandatory when using SpriteUtils
            self.actives_group = SPInit(self.screen, self.backgr)
            self.actives_group.set_onematch(True)
            # MustSPMainCore.py be called before using SPWidgets
            Init(self.theme)
            # and reset the screen used in spgoodies
            self.spgoodies.screen = self.screen
        else:
            theme_dir = self.spgoodies.get_home_theme_path()
            xmlname = 'SP_menu.xml'
        session.close()
        # We get the build menu activity here.
        # check if we have local_quizdata, if not we remove the menu button
#        orm, session = self.dm.get_orm('game_quizregional', 'content')
#        row = session.query(orm).first()
#        if row:
#            showlocalquiz = True
#        else:
#            showlocalquiz = False
#        session.close()
#        orm, session = self.dm.get_orm('game_quizpersonal', 'content')
#        row = session.query(orm).filter_by(user_id = user_id).first()
#        if row:
#            showpersonalquiz = True
#        else:
#            showpersonalquiz = False
#        session.close()
        showlocalquiz = False
        showpersonalquiz = False
        try:
            self.activity = SPMenu.Activity(self.COPmode, showpersonalquiz, showlocalquiz)
        except Exception, info:
            self.logger.exception("Failed to setup the menu")
            raise utils.MyError, info
        # First we parse the menu xml            
        self.activity._parse_menu(os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', self.theme), xmlname)
        # now we can build the menu buttons
        if self.COPmode:
            theme_dir = os.path.join('controlpanel_lgpl','lib', 'SPData')
        else:
            theme_dir = os.path.join(THEMESPATH, self.theme)
        self.activity._build_menu(theme_dir, self.theme_rc, language[0])
        
        r = pygame.Rect(0, blitpos_y, 800, 500)
        # Add the rect to SPGoodies so that activities know where to blit.
        self.spgoodies.screenrect = r
        self.spgoodies.background = self.backgr
               
        # now we can finish the menu object
        self.activity._setup(self.spgoodies, self.dm, self.theme_rc['menubarbottom'])
        
        # Setup the menubar
        self.excludebuttons = self.theme_rc['exclude_buttons']
        if self.theme_rc['level_indicator'] == 'star':
            usestar = True
        else:
            usestar = False
        if 'graph' in self.theme_rc['exclude_buttons']:
            usegraph = False
        else:
            usegraph = True
            
        self.menubar = MenuBar(self.menu_rect, \
                               self.actives_group, self.menubar_cbf,\
                                self.menubar_dice_cbf, self.lock, usestar, \
                                usegraph, volume_level=self.volume_level)
        self.infobutton = self.menubar.get_infobutton()
        if self.COPmode:
            self.infobutton.moveto((CORE_BUTTONS_XCOORDS[7], self.menubar.get_buttons_posy()))
        self.quitbutton = self.menubar.get_quitbutton()
        self.scoredisplay = self.menubar.get_scoredisplay()
        self.volumebutton = self.menubar.get_volumebutton()
        self.menubarbuttons = [self.infobutton, self.quitbutton, self.volumebutton]
        
        self.chartbutton = self.menubar.get_chartbutton()
        self.dicebuttons = self.menubar.get_dicebuttons()
                
        self.spgoodies.scoredisplay = self.scoredisplay
        self.spgoodies.infobutton = self.infobutton
        self.spgoodies.quitbutton = self.quitbutton
        
        self.spgoodies.dm = self.dm
        
        if not 'dice' in self.excludebuttons and not self.COPmode:
            self.dicebuttons.UseDice = True
        else:
            self.dicebuttons.UseDice = False
        if not 'graph' in self.excludebuttons and not self.COPmode:
            self.menubarbuttons.append(self.chartbutton)
            # disable the chartbutton
            self.chartbutton.enable(False) 
        if not 'score' in self.excludebuttons and not self.COPmode:
            self.menubarbuttons.append(self.scoredisplay)
        
        self.actives_group.add(self.menubarbuttons)
        
        # add the current user, if it exists
        text = _("User: %s") % self.dm.get_username()
        # TODO: Do we do an andmin mode ?
        if self.cmd_options.adminmode:
            text = text + " (Adminmode)"
        if self.theme_rc['menubar_position'] == 'bottom':
            y = 500
        else:
            y = 0
        if not 'user' in self.excludebuttons:
            self.usernamelabel = Label(text, (CORE_BUTTONS_XCOORDS[1], y+10))
        else:
            self.usernamelabel = None
        if not 'activity' in self.excludebuttons:
            text = _("Activity :") 
            self.activitylabel = Label(text, (CORE_BUTTONS_XCOORDS[1], y+50))
        else:
            self.activitylabel = None
        
        # Show the mainscreen and menubar
        self.clear_screen()
        
        # setup misc stuff
        self.clock = pygame.time.Clock()
        self.run_activity_loop = True
        self.run_event_loop = True
        self.run_main_loop = True
        self.framerate = 30
        self.were_in_pre_level = False
        self.store_data = True
                        
        if self.COPmode:
            self.clear_screen()
            self.disable_menubuttons()
            self.scoredisplay.UseMe = False
            self.enable_exit_button()
            self.quitbutton.display_sprite()
    
    def register_foreign_observer(self, obs):
        """You should register your observer that will take care of any cleanup
        in case of an error"""
        self.foreign_observers.append(obs)
    
    def call_foreign_observers(self):
        self.logger.warning("Calling foreign_observers to cleanup their own mess")
        for obs in self.foreign_observers:
            apply(obs)
    
    def start(self):
        # start menu
        self.activity.start()
        #For memoryleak debugging 
        #h.setrelheap()
        self.start_main_loop()
    
    def get_mainscreen(self):
        return self.screen

    def get_current_user(self):
        return self.dm.get_username()

    ############################################################################
    ## Methods for internal use 
    def clear_screen(self):
        self.logger.debug("clear_screen called")
        # rebuild screen with the standard background 
        x, y = self.spgoodies.screenrect.left, self.spgoodies.screenrect.top
        pygame.display.update(self.screen.blit(self.backgroundimage, (0, 0)))
        self.backgr.blit(self.screen.convert(),(0, 0) )
        # reset the screens in SPGoodies so that any activities gets a proper screen.
        self.spgoodies.screen = self.screen
        self.spgoodies.background = self.backgr
        if self.COPmode:
            self.quitbutton.display_sprite()
            if self.activity.get_name() != 'menu':
                self.enable_info_button()
                self.infobutton.display_sprite()
            else:
                self.disable_info_button()
            return
        for b in self.menubarbuttons:
            b.display_sprite()
        if self.dicebuttons:
            #self.dicebuttons.next_level(self.levelcount)
            self.dicebuttons.show()
        if self.usernamelabel:
            self.usernamelabel.display_sprite()
        if self.activitylabel:
            self.activitylabel.display_sprite()
    
    def load_activity(self, name):
        # This will replace self.activity with the chosen activity
        self.logger.debug("load_activity called with %s" % name)
        
        self.set_framerate()
        try:
            if self.COPmode:
                p = os.path.join('controlpanel_lgpl', 'lib')
            else:
                p = 'lib'
            self.activity_module = utils.import_module(os.path.join(BASEDIR, p, name))
            self.activity = self.activity_module.Activity(self.spgoodies)
            if self.COPmode:
                if self.activity.get_name() == 'menu':
                    self.disable_info_button()
                else:
                    self.enable_info_button()
                    self.infobutton.display_sprite()
        except (utils.MyError, ImportError, AttributeError), info:
            self.logger.exception("Error importing activity %s" % name)
            dlg = Dialog(_("Error importing activity\n%s" % info),dialogwidth=500,  \
                    buttons=[_("OK")], title=_('Error !'))
            dlg.run()
            self.activity = self.menu_saved
            self.activity.refresh_sprites()
            self.start_main_loop()
        except (utils.MyError, StandardError), info:
            self.logger.exception("Error constructing activity: %s" % info)
            dlg = Dialog(_("Error constructing activity\n%s" % info), dialogwidth=500, \
                    buttons=[_("OK")], title=_('Error !'))
            dlg.run()
            self.activity = self.menu_saved
            self.activity.refresh_sprites()
            self.start_main_loop()
        else:
            self.logger.debug("Loaded %s activity succesfull" % self.activity)
            text = _("Activity : %s") % self.activity.get_helptitle()
            if self.activitylabel:
                self.activitylabel.settext(text)
            if self.activity.get_name() == 'dltr':
                self.logger.debug("Got dltr, putting myself in DT mode")
                self.DTmode = True
                self.DTactivity = self.activity
                mapper = self.dm.get_mapper(self.activity.get_name())
                self.DTactivity._set_up(mapper)
                self.hide_level_indicator()
                self.enable_dice(False)
            elif self.activity.get_name() == 'quiz_general':
                self.logger.debug("Got the general, putting myself in DT mode")
                self.DTmode = True
                self.DTactivity = self.activity
                mapper = self.dm.get_mapper(self.activity.get_name())
                self.DTactivity._set_up(mapper)
                self.DTlevelcount = 1
            else:
                self.enable_dice(True)
            self.statshash['datetime'] = datetime.datetime.now()
            self.statshash['activity_name'] = name
            self.logger.debug("Resetting all sqla sessions")
            self.dm.reset()
            self.start_main_loop()
            
    def start_start(self):
        self.logger.debug("Calling activity start")
        self.statshash['game_start_called'] = True
        self.show_level_indicator()
        self.levelcount = 1
        self.store_data = True
        try:
            self.activity.start()
        except Exception, info:
            self.logger.exception("Error in %s start" % self.activity.get_name())
            raise utils.MyError, info
        if 'graph' not in self.excludebuttons:
            # as were datacollectors we also add a chart button to the menubar
            self.chartbutton.enable(True)
            
    def display_levelstart(self, level):
        """Displays a 'ready 3-2-1' screen prior to starting the level.
        It also displays a blocking 'Start' button."""
          
        self.logger.debug("Starting display_levelstart")
        
        if self.cmd_options.no_text or self.cmd_options.nocountdown or self.DTmode:
            return
                
        org_screen = pygame.display.get_surface().convert()
        org_backgr = self.backgr.convert()

        self.screen.set_clip()
        
        # disable the dice to prevent users clicking it
        # dim the screen
        sdim = utils.Dimmer(keepalive=0)
        sdim.dim(darken_factor=200, color_filter=(51, 10, 10))
        # display text
        txt = _("Starting level %s") % level
        txt_s0 = utils.char2surf(txt, TTFSIZE + 20, fcol=GREEN, ttf=TTF, bold=True)
        self.screen.blit(txt_s0, ((780-txt_s0.get_width()) / 2, 100))
         
        txt = self.activity.get_help()[1]
        txt_s1 = utils.char2surf(txt, TTFSIZE + 2, fcol=GREEN, ttf=TTF, bold=True)
        self.screen.blit(txt_s1, ((780-txt_s1.get_width()) / 2, 200))
        
        txt = utils.txtfmt([_("Hit the 'space' key or a mousebutton to skip the countdown")], 60)
        txtlist = []
        for line in txt:
            txtlist.append(utils.char2surf(line, TTFSIZE + 2, fcol=GREEN, ttf=TTF, bold=True))
        y = 260
        for s in txtlist:
            self.screen.blit(s, (40, y))
            y += s.get_height()
        
        pygame.display.update()
        
        # we have a animated sprite so we need to adjust the backgr screen
        self.backgr.blit(self.screen, (0, 0))
        # we re-init the sprite stuff as we use a different background 
        SPInit(self.screen, self.backgr)
        # i is used as a counter. The loop runs at approx 30 times a second so
        # when i == 30 we decrease the counter.  
        i = 0
        counter = 4# used for 4-3-2-1
        spr = None# will hold the sprite
        # local eventloop
        clock = pygame.time.Clock()
        runloop = True
        while runloop:
            # runloop is set to False by the a event or when the counter == 0
            pygame.event.pump()
            events = pygame.event.get()
            for event in events:
                if event.type is KEYDOWN and event.key is K_SPACE or\
                    event.type is MOUSEBUTTONDOWN:
                    runloop = False
                    break
            if i >= 29:
                if spr:
                    spr.erase_sprite()
                if counter == 0:
                    runloop = False
                else:
                    num_s = utils.char2surf(str(counter), 72, fcol=RED, ttf=TTF, bold=True)
                    spr = MySprite(num_s)
                    spr.display_sprite((350, 300))
                    i = 0
                    counter -= 1
            i += 1
            clock.tick(30)
        # restore the screens
        # undim the screen
        sdim.dim(darken_factor=255, color_filter=(0, 0, 0))
        self.screen.blit(org_screen, (0, 0))
        self.backgr.blit(org_backgr, (0, 0))
        # reset the screens in SPGoodies so that any activities gets a proper screen.
        SPInit(self.screen, self.backgr)
        pygame.display.update()
                
    def start_next_level(self, store_db=None, nextlevel=None, levelup=False, no_question=False):
        """store_db is used to signal normal ending and that we should store the data"""
        self.logger.debug("start_next_level called with: %s, %s, %s,%s" % (store_db,nextlevel,levelup, no_question))
        self.statshash['game_nextlevel_called'] = True
        self.clear_screen()
        self.screen.set_clip()
        if nextlevel < 1:
            nextlevel = 1
        if nextlevel + (self.dicebuttons.get_minimal_level() - 1) > 6:
            nextlevel = 6 - (self.dicebuttons.get_minimal_level() - 1)
            self.logger.info("activity %s only has %s level, reset nextlevel to %s" % (self.activity.get_name(),\
                                                                                       self.dicebuttons.get_minimal_level() + 1,\
                                                                                       nextlevel))
        if nextlevel and self.activity.get_name() != 'dltr':
            if levelup and (self.levelcount + self.dicebuttons.get_minimal_level() -1) < 6:
                if not no_question and not self.no_levelup_dlg:
                    dlg = Dialog(_("You play very good, do you want to try the next level ?"), \
                            buttons=[_("No"), _("Yes")], title=_('Question ?'), dialogwidth=300)
                    dlg.run()
                    c = dlg.get_result()[0]
                else:
                    c = _('Yes')
                if c == _('Yes'):
                    self.levelcount = nextlevel
            else:
                self.levelcount = nextlevel
        if nextlevel and self.DTactivity and self.DTactivity.get_name() == 'dltr':
            self.DTlevelcount = nextlevel
        # are we in the data collecting bussiness ?
        if self.activity.get_name() != 'menu':
            # Here we fetch our data colletor object, we collect data per level,
            # so it's the best place to start datacollecting
            # get_mapper returns always an object, in case of anonymous mode the 
            # object is a fake.
            self.mapper = self.dm.get_mapper(self.activity.get_name())
            self.mapper.insert('level', self.levelcount)
            self.mapper.insert('start_time', utils.current_time())
        else:
            self.mapper = self.dm.get_mapper(None)
        try:
            if self.activity.get_name() == 'menu':
                self.activity.next_level(self.levelcount)
            else:
                if not self.menubuttons_backup:
                    if self.DTmode and self.DTactivity.get_name() == 'dltr':
                        level = self.DTlevelcount + (self.dicebuttons.get_minimal_level() -1)
                    else:
                        level = self.levelcount
                    
                if self.activity.get_name() not in ('dltr', 'quiz_general') and self.DTmode:
                    if self.DTlevelscounter > 0:
                        self.DTactivity.refresh_sprites()
                        self.logger.debug("Calling %s dailytraining_next_level, level:%s,counter:%s" % \
                                          (self.activity.get_name(), self.DTlevelcount, self.DTlevelscounter))
                        result = self.activity.dailytraining_next_level(self.DTlevelcount, self.mapper)
                        self.DTlevelscounter -= 1
                    else:
                        self.DTact_is_finished = True
                        # we don't store the scores as it differs from normal act results
                        # the DT act stores the results
                        self.activity_game_end(store_db=False)
                        result = True
                else:
                    self.logger.debug("Calling %s next_level, level:%s" % (self.activity.get_name(), self.levelcount))
                    result = self.activity.next_level(self.levelcount, self.mapper)
                # we pass the db mapper so that the activity can store it's data
                if not result:
                    if self.DTmode:
                        self.logger.debug("End the DT")
                        self.DTmode = False
                        self.DTactivity = None
                    self.logger.debug("start_activity_loop: Levels ended")
                    #self.activity_info_dialog(self.activity.get_helplevels())
                    self.activity_game_end(store_db=False)
                else:
                    # We display a blocking ready-start screen
                    self.display_levelstart(self.levelcount)
                    # set the level indicator
                    self.dicebuttons.next_level(self.levelcount)
                    # and call the post_next_level
                    self.activity.post_next_level()
        except MainEscapeKeyException:
            self.activity.stop_timer()
            raise MainEscapeKeyException
        except utils.MyError, info:
            self.logger.error("MyError in %s next_level" % self.activity.get_name())
            self.activity.stop_timer()
            raise utils.MyError, info
        except StandardError, info:
            self.logger.exception("StandardError in %s next_level" % self.activity.get_name())
            self.activity.stop_timer()
            raise utils.MyError, info
        except Exception, info:
            self.logger.exception("Exception in %s next_level" % self.activity.get_name())
            self.activity.stop_timer()
            raise utils.MyError, info
        
        
    def ask_exit(self):
        self.logger.debug("ask_exit called")
        if not self.cmd_options.noexitquestion:
            dlg = Dialog(_("Do you really want to quit ?"), fsize=20, dialogwidth=300,\
                    buttons=[_("Cancel"), _("OK")], title=_('Quit ?'))
            dlg.run()
            c = dlg.get_result()[0]
            self.activity.refresh_sprites()
        else:
            c = _("OK")
        if c == _("OK"):
            self.logger.info("User wants exit")
            if self.activity.get_name() != 'menu':
                try:
                    self.activity_game_end(store_db=False)
                except:
                    pass
            # any cleanup is done by atexit functions
            # except this one :-)
            self.dm._cleanup()
            # dim the screen
            sdim = utils.Dimmer(keepalive=0)
            sdim.dim(darken_factor=200, color_filter=(51, 10, 10))
            # display text
            txt = utils.txtfmt([_("Stopping timers, please wait...")], 50)
            y = 240
            for line in txt:
                s = utils.char2surf(line, TTFSIZE + 12, fcol=GREEN, ttf=TTF, bold=True)
                self.screen.blit(s, (10, y))
                y += s.get_height()
            pygame.display.update()
            self.store_stats()
            #pygame.time.wait(1000)# just to show the text in case there are no timers
            raise MainEscapeKeyException # let schoolsplay.py decide what next
    
    def store_stats(self):
        orm, session = self.spgoodies.get_orm('stats', 'user')
        session.query(orm)
        session.add(orm(**self.statshash))
        session.commit()
        session.close()
        
    ###########################################################################
    # Main menubar button callbacks.
    # These are called when the user hits the buttons in the lower menu bar of 
    # screen.
    def menubar_cbf(self, sprite, event, data):
        """Callback function for the menubar buttons: quit, info, chart.
        Here we decide which button has called us and we call the appropriate 
        method for further event handling."""
        self.logger.debug("menubar_button_pressed: %s" % sprite.name)
        if hasattr(self.activity, 'stop_sound'):
            self.activity.stop_sound()
        try:
            if sprite.name == 'Info':
                self.core_info_button_pressed()
                sprite.mouse_hover_leave()
            elif sprite.name == 'Quit':
                sprite.mouse_hover_leave()
                if self.DTmode and self.DTactivity.get_name() == 'dltr':
                    self.lock.acquire()
                    text = _("Are sure you want to quit the dailytraining ? All the results will be lost.")
                    dlg = Dialog(text, buttons=[_("No"), _("Yes")], title=_('Question ?'), dialogwidth=300)
                    dlg.run()
                    c = dlg.get_result()[0]
                    if c == _('No'):
                        self.lock.release()
                        return
                raise utils.StopGameException
                #self.core_quit_button_pressed()
            elif sprite.name == 'Chart' and self.activity.get_name() != 'menu':
                self.core_chart_button_pressed()
                sprite.mouse_hover_leave()
            elif sprite.name == 'Volume':
                self.core_volume_button_pressed()
                self.volumebutton.mouse_hover_leave()
        except StandardError, info:
            self.logger.exception("System error: %s" % info)
            self.lock.release()
            self.clear_screen()
            self.activity.refresh_sprites() 
        
    def menubar_dice_cbf(self, sprite, event, data):
        """Callback function for the menubar dicebutton."""
        self.logger.debug("core_dice_button_pressed: %s" % data)
        if hasattr(self.activity, 'stop_sound'):
            self.activity.stop_sound()
        self.myspritesactivated = True
        if self.DTmode and self.DTactivity.get_name() != 'dltr':
            self.DTactivity.act_hit_levelindicator(data)
            self.DTlevelcount = data
        if data:
            if 1 > data > 6:
                data = 1
            if self.were_in_pre_level:
                self.levelcount = data
            else:
                self.activity_level_end(nextlevel=data, score_audit=False)
        else:
            self.activity.refresh_sprites()

    def core_info_button_pressed(self):
        self.myspritesactivated = True
        self.lock.acquire()
        # Construct the help text from the activity's various get_help* methods
        help = self.activity.get_help()
        if self.activity.get_name() != 'menu' and not self.COPmode:
            tip = self.activity.get_helptip()
            if tip:
                help.append(_("Tip:%s") % tip[0])
            help.append(self.activity.get_helplevels())
            #help.append(_("This activity belongs to the category: %s") % self.activity.get_helptype())
        if hasattr(self.activity, 'get_extra_info'):
            help.append("%s" % self.activity.get_extra_info())
        buttons = [_("OK")]
        
        if hasattr(self.activity, 'get_moviepath'):
            if os.path.exists(self.activity.get_moviepath()):
                buttons.append(_("Movie"))
        # make sure all lines can fit the screen
        newlines = utils.txtfmtlines(help, 80)
        dlg = Dialog(newlines, dialogwidth=600, buttons=buttons, \
                                            title= _("Information about %s") % 
                                            self.activity.get_helptitle())
        dlg.run()
        if dlg.result[0] == _("Movie"):
            # play movie
            self.logger.debug("Start videoplayer")
            vpl = SPVideoPlayer.Player(os.path.join(THEMESPATH, self.theme,'btp_800x600.vlt'))
            mv = self.activity.get_moviepath()
            self.logger.debug("playing %s" % mv)
            result = vpl.start(mv)
            if not result[0]:
                self.logger.error("Failed to start player, %s" % result[1])
        self.activity.refresh_sprites()
        self.lock.release()
        return True
  
    def core_volume_button_pressed(self):
        self.myspritesactivated = True
        self.lock.acquire()
        
        dlg = Dialog('',dialogwidth=300, dialogheight=300,  \
                    buttons=[_("Close")], title=_('Set volume level'))
        
        r = dlg.get_action_area()['action_area']
        x, y = r.topleft
        x += 50
        y += 30
        va = VolumeAdjust((x, y), volume=int(self.volume_level), voice_unmute=self.spgoodies._unmute_quiz_voice)
        va.set_use_current_background(True)# it will use the current screen as background as we blit over a dialog
        va.display()
        dlg.run(va.get_actives())
        self.volume_level = va.get_volume()
        self.spgoodies._unmute_quiz_voice = va.get_voice_state()
        if hasattr(self.activity, 'quiz_voice_changed'):
            self.activity.quiz_voice_changed(self.spgoodies._unmute_quiz_voice)
        self.logger.debug("quiz voice unmute: %s" % self.spgoodies._unmute_quiz_voice)
        pos = (CORE_BUTTONS_XCOORDS[7], self.menu_rect.top+8)
        self.volumebutton.erase_sprite()
        self.actives_group.remove(self.volumebutton)
        if self.volume_level != 0:
            bname = 'core_volume_button.png'
            hbname = 'core_volume_button_ro.png'
            p = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', self.theme, bname)
            hp = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', self.theme, hbname)
            self.volumebutton = TransImgButton(p, hp,pos, name='Volume')
        else:
            bname = 'core_volmute_button.png'
            hbname = 'core_volmute_button_ro.png'
            p = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', self.theme, bname)
            hp = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', self.theme, hbname)
            self.volumebutton = TransImgButton(p, hp,pos, name='Volume')
        self.volumebutton.connect_callback(self.menubar_cbf, MOUSEBUTTONUP, 'Volume')    
        self.actives_group.add(self.volumebutton)
        self.menubarbuttons[2] = self.volumebutton
        self.logger.info("setting volume to %s" % self.volume_level)
        orm, session = self.get_orm('users', 'user')
        query = session.query(orm).filter_by(user_id = self.dm.get_user_id())
        query.update({orm.audio:self.volume_level}, synchronize_session=False)
        session.commit()
        session.close()
        self.lock.release()
        return True
        
    # Callback for the chart button in the menubar. Passed to SPCoreButtons.ChartButton
    def core_chart_button_pressed(self):
        if not hasattr(self, 'mapper'):
            self.logger.warning("core_chart_button_pressed, no mapper found.")
            return 
        self.myspritesactivated = True
        self.lock.acquire()
        if self.DTmode and self.DTactivity.get_name() == 'dltr':
            mapper = self.DTactivity.dbmapper
            name = self.DTactivity.get_name()
        else:
            mapper = self.mapper
            name = self.activity.get_name()
        # get data, user AND level
        query = mapper._get_level_data(levelnum=self.levelcount)
        # get the score and date data from the objects
        scorelist = filter(None, [(getattr(item, 'start_time'), getattr(item, 'score')) for item in query])
        self.logger.debug("scorelist from SQL query: %s" % scorelist)
        # get the mu and sigma for this activity.
        norm = self.dm.get_mu_sigma(self.activity.get_name())
        # apply the Z-TRANSFORMATION as described in the post:
        # [Schoolsplay-devel] score mathematic
        # See also the "z - Score (zS)" at http://cse.yeditepe.edu.tr/~eozcan/webTools/grading/stat.html
        prclist = utils.score2percentile(scorelist, norm[0], norm[1])
        self.logger.debug("percentile list: %s" % prclist)
        # turn the data into a graph blitted on a SDL surface
        htext = _("Generated by %s version %s for the %s activity") % (self.theme, Version.version, name)
        s = Graph(prclist, self.levelcount, norm=norm, headertext=htext)
        dlg = Dialog(s, buttons=[_("OK")], title=_('Results'))
        dlg.run()
        
        result = dlg.get_result()
        if result[0] == _('Save image'):
            name = self.activity.get_name()
            user = self.dm.get_username()
            level = self.levelcount
            path = os.path.join(HOMEDIR, '%s-%s-level_%s.BMP' % (user, name, level))
            pygame.image.save(s.get_surface(), path)
            dlg = Dialog(_("Your image is saved at:\n%s" % path),dialogwidth=500, \
                       buttons=[_("OK")],  title=_('Information'))
            dlg.run()
        self.activity.refresh_sprites()
        # and release the lock again
        self.lock.release()
        
    #############################################################################
    # SPGoodies callbacks that are passed to the activity and are used as observers
    # by the maincore. The activity calls one of these callbacks from within the
    # activity_loop. This means that when we don't set the
    # run_*_loop to False we will return back to the activity_loop.
    # The whole communication between the core and the activity is based on the
    # observer pattern. The dbase mapper object is not part of SPGoodies but is passed
    # in the activity.start method.
    def disable_menubuttons(self):
        """Activity wants to disable the menu buttons.
        We enable the buttons when the activity ends."""
        self.logger.debug("disabled menubar buttons")
        self.menubuttons_backup = self.actives_group.sprites()
        self.enable_dice(False)
        for s in self.menubuttons_backup:
            s.erase_sprite()
        self.actives_group.empty()
    def enable_menubuttons(self):
        """Activity wants to enable the menu buttons again."""
        if self.menubuttons_backup and not self.COPmode:
            self.logger.debug("enable menubar buttons")
            for s in self.menubuttons_backup:
                s.display_sprite()
            self.actives_group.add(self.menubuttons_backup)
            self.menubuttons_backup = []
    def disable_level_indicator(self):
        self.dicebuttons.enable(False)
    def enable_level_indicator(self):
        self.dicebuttons.enable(True)
    def disable_exit_button(self):
        self.actives_group.remove(self.quitbutton)
    def enable_exit_button(self):
        self.actives_group.add(self.quitbutton)
    def disable_score_button(self):
        self.actives_group.remove(self.scoredisplay)
    def enable_score_button(self):
        self.actives_group.add(self.scoredisplay)
    def hide_level_indicator(self):
        self.dicebuttons.erase()
    def show_level_indicator(self):
        self.dicebuttons.show()
    def disable_info_button(self):
        self.actives_group.remove(self.infobutton)
    def enable_info_button(self):
        self.actives_group.add(self.infobutton)
    def get_orm(self, tname, dbase):
        self.logger.debug("get_orm called with %s, %s" % (tname, dbase))
        return self.dm.get_orm(tname, dbase)
    
    def activity_pre_level_end(self):
        """The pre_level is ended.
        The core will call next_level on the activity.
        """
        self.logger.debug("activity_pre_level_end called, stopping eventloop")
        self.run_event_loop = False
    
    def activity_level_end(self, store_db=None, nextlevel=1, levelup=False, no_question=False, score_audit=True):
        """The level is ended.
        The core will call next_level on the activity.
        @store is used by the activity to signal a normal level end."""
        self.logger.debug("activity_level_end called with: %s,%s,%s,%s" % \
                          (store_db, nextlevel, levelup, no_question))
        # reset any clipping
        self.screen.set_clip()
        
        sc = False
        if hasattr(self,'mapper') and str(self.mapper) != "BogusMapper" and self.activity.get_name() != 'quiz_general':
            self.logger.debug("store dbase data for %s" % self.activity.get_name())
            self.mapper.insert('end_time', utils.current_time())
            # calculate the time spend inside the level and score
            stime = utils.calculate_time(self.mapper._get_start_time(), \
                self.mapper._get_end_time())
            self.mapper.insert('timespend', stime)
            sc = self.activity.get_score(stime)
            self.logger.debug('got activity score: %s' % sc)
            self.statshash['final_score'] = sc
            self.store_stats()
            self.mapper.insert('score', sc)
        if not self.DTactivity and score_audit:
            if sc < 5.0:
                self.logger.debug('score < 5.0 or None, set levelup to false')
                if nextlevel > 1:
                    self.logger.debug('decrease nextlevel value with one')
                    nextlevel -= 1
                    if nextlevel < 1:
                        nextlevel = 1
                levelup = False
            elif 5 < sc < 7:
                self.logger.debug('set levelup to false')
                levelup = False
            else:
                if nextlevel < 6 and levelup:
                    self.logger.debug("Increase nextlevel with one")
                    nextlevel += 1
        self.logger.debug("activity_level_end final values:  %s,%s,%s,%s" % \
                          (store_db, nextlevel, levelup, no_question))
        if store_db and sc and sc <= 10:# If we don't get a score we store nothing.
            # You MUST call commit to write the contents of the mapper to disk.
            self.mapper.commit()
        else:
            self.logger.debug("not storing or score > 10, nothing committed to dbase")
            self.mapper.close()
        
        if self.DTmode:
            self.DTactivity.act_next_level_stopped(sc)
        # we try to stop any timers or execounter, just in case
        try:
            self.activity.stop_timer()
        except:
            pass
        try:
            self.execounter.erase()
        except Exception:
            pass
        # add a little pause between levels.
        if not self.cmd_options.no_level_pause:
            p = os.path.dirname(self.core_rc_path)
            if self.core_rc.has_key('level_change_sound') and self.core_rc['level_change_sound']:
                snd = utils.load_music(os.path.join(p, self.core_rc['level_change_sound']))
                snd.play()
            if self.core_rc.has_key('level_change_image') and self.core_rc['level_change_image']:
                img = utils.load_image(os.path.join(p, self.core_rc['level_change_image']))
                self.screen.blit(img, (275, 200))
                pygame.display.update()
            if not self.core_rc.has_key('level_change_pause'):
                pause = 0
            else:
                pause = int(self.core_rc['level_change_pause'])
            if pause: 
                pygame.time.wait(pause*1000)
        self.clear_screen()
        self.start_next_level(store_db=store_db, nextlevel=nextlevel, levelup=levelup, no_question=no_question)
                
    def activity_game_end(self, store_db=True):
        """Tthe game is ended.
        The core will start the menu, delete the activity and cleans the tmp dir.
        @store is used by the activity to signal a normal level end."""
        self.logger.debug("activity_game_end called")
        self.statshash['game_end_called'] = True
        self.store_stats()
        try:
            self.lock.release()# release any locks that were set prior to this
        except Exception, info:
            #print info
            pass
        try:
            self.chart_but.erase()
        except Exception, info:
            #print info
            pass
        try:
            self.activity.stop_timer()
        except Exception, info:
            #print info
            pass
        try:
            self.activity.stop()
        except Exception,info:
            #print info
            pass
        try:
            self.execounter.erase()
        except Exception, info:
            #print info
            pass
        self.levelcount = 1
        # reset framerate
        self.set_framerate()
        # datacollecting
        if not self.activity.get_name() == 'menu' and store_db and hasattr(self, 'mapper'):
            self.logger.debug("commiting data to dbase")
            self.mapper.commit()# write any sql stuff to disk
        else:
            self.logger.debug("Not commiting data to dbase")
        # check if we are the menu, if so we in deep shit and theres a big problem
        # in the menu
        if self.activity and self.activity.get_name() == 'menu':
            self.logger.critical("I'm the menu, I shouldn't be here, bail out")
            self.logger.exception("Check the logs, there's a problem in the menu")
            sys.exit(1)
#        # It seems that Python doesn't free all the memory like expected.
#        # we must delete the activity to prevent memoryleak.
#        # explicitly set the ref count to 0 for the activity
#        del self.activity
#        # And tell the garbage collector to free the memory
#        gc.collect()
        self.dicebuttons.enable(False)
        
        if self.DTmode and self.DTact_is_finished:
            self.DTactivity.act_stopped(self.activity)
            self.DTactivity.dt_restarting()
            self.logger.debug("Restarting DT")
            self.activity = self.DTactivity
            self.DTact_is_finished = None
        else:
            # Now we can start the menu
            self.logger.debug("Restarting menu")
            self.activity = self.menu_saved
            self.DTactivity = None
            self.DTmode = None
            self.set_dice_minimal_level(1)
            self.levelcount = 1
            if os.path.exists(os.path.join(TEMPDIR, 'previous_screenshot.jpeg')):
                os.remove(os.path.join(TEMPDIR, 'previous_screenshot.jpeg'))
        text = _("Current activity : %s") % self.activity.get_helptitle()
        if self.activitylabel:
            self.activitylabel.settext(text)
            self.activitylabel.display_sprite()
        self.run_main_loop = True
        #self.clear_screen()
        self.start_main_loop()
        
    def activity_info_dialog(self, text):
        self.logger.debug("activity_info_dialog called")
        dlg = Dialog(text,buttons=[_('OK')], title=_('Information'), dialogwidth=300)
        dlg.run()
        if hasattr(self, 'activity'):
            self.activity.refresh_sprites()

    def display_execounter(self, total):
        self.logger.debug("display_execounter called")
        self.execounter = ExeCounter((CORE_BUTTONS_XCOORDS[5], 8), total)
        return self.execounter
        
    # Creates a virtual keyboard gui to be used when running in kioskmode
    def get_virtkb(self):
        """return a virtual keyboard object.
        As this takes some time to create you should not destroy it but
        calls it's hide method to hide it.
        """
        # do we already have a kb?
        if self.vtkb:
            return self.vtkb
        # are we in kioskmode?
        if not self.cmd_options.kioskmode:
            return None
        # setup a new vt kb
        pos = (250, 480)
        fsize = 22
        vkb = VirtualKeyboard(self.screen, fsize, pos)
        self.vtkb = vkb
        return self.vtkb
    
    def set_framerate(self, rate=30):
        """Used by activities to set a lower framerate. Only use this if you
        know why you want to alter the framerate.
        The core resets the framerate to 30 fr/min when loading a activity.
        """
        self.logger.debug("set_framerate called with rate %s" % rate)
        if rate > 30:
            rate = 30
            self.logger.critical("framerate set to %s, which is not allowed as it could run the CPU to 100%" % rate)
        self.framerate = rate

    def enable_dice(self, enable=True):
        if self.menubuttons_backup or self.COPmode or self.DTmode and self.DTactivity.get_name() == 'dltr':
            self.logger.debug("menubuttons disabled or we in COPmode, not enabling dice")
            return
        self.logger.debug("enable_dice called with: %s" % enable)
        self.dicebuttons.enable(enable)

    def set_dice_minimal_level(self, level):
        self.dicebuttons.set_minimal_level(level)
        
    # Added to the spgoodies for the menu and dltr activity
    def _menu_activity_userchoice(self, activity, level=None, cycles=None):
        self.logger.debug("menu_activity_userchoice called with act:%s, level:%s, cycles:%s"\
                           % (activity, level, cycles))
        if self.DTmode:
            self.DTlevelcount = level
            self.DTlevelscounter = cycles
        else:
            self.menu_saved = self.activity
        self.load_activity(activity)
        
    
    ############################################################################
    # Here we start the loop. This loop will call activity.start and the calls 
    # the activity eventloop.
    #
    # The way to get out of the start_main_loop is when the user hits the escape
    # key which will raise an exception that we catch in start_main_loop.
    # We will ask to really quit or not, if not we don't break but call the
    # activity_loop again. The hitting of the quit button works the same as
    # the escape key.
    #
    # The other way to get out of activity_loop is when the activity level is ended
    # then the activity calls the callback: activity_level_end (see above)
    # The activity is passed a class object called SPGoodies which holds a number
    # of 'observer' methods which calls methods in the core. So when we talk about
    # the activity that calls callbacks, we mean SPGoodies observers.
    # This callback will determine what's next, but in fact were now out of the
    # start_main_loop. If there are more levels the activity_level_end method
    # will call the activity_loop again. 
    #
    # Finally we can break out when the activity is ended because all levels are done.
    # Then the activity will call the callback: activity_game_end (see above)
    # This callback will call activity.cleanup() to force any cleanup by the activity
    # and delete the activity. Then it will restart the menu and call the start_main_loop
    # which will run the menu again.
    #
    # In short:
    # The menu is constructed and start_main_loop is called.
    # Menu is started and the activity_loop is called which will loop the menu loop
    # until the user hits a button.
    # Then the menu will call SPGoodies.menu_activity_userchoice which will end
    # the menu and start the choosen activity.
    # Then the start_main_loop is called by menu_activity_userchoice and the 
    # activity is started and the activity_loop is called. This will loop until
    # the activity calls one of the SPGoodies callbacks which will jump out the loop
    # and determine what's next. 
    def start_main_loop(self):
        error = False
        store = True
        # we only break this loop on SP quit or escape key
        # The menu activity starts this also from _menu_activity_userchoice
        while self.run_main_loop:
            if self.activity.get_name() != 'menu':
                self.logger.debug("Start activity start")
                try:
                    self.start_start()
                except (utils.MyError, utils.StopGameException), info:
                    self.logger.error("%s" % info)
                    self.run_event_loop = False
                    self.run_activity_loop = False
                    self.run_main_loop = False
            self.run_activity_loop = True
            try:
                self.logger.debug("Start activity pre_level_loop")
                self.start_pre_level_loop()
            except EscapeKeyException:
                # user hits escape
                self.logger.info("User hits escape")
                self.ask_exit()# if this returns the user said no.
                self.store_data = False
            except utils.StopGameException:
                self.logger.debug("User hits quit menubutton")
                self.run_event_loop = False
                self.run_activity_loop = False
                self.run_main_loop = False
                self.store_data = False
            except utils.MyError, info:
                # trouble in paradise
                self.logger.error("start_main_loop: MyError, %s" % info)
                self.run_event_loop = False
                self.run_activity_loop = False
                self.run_main_loop = False
                error = True
                self.store_data = False
            except StandardError, info:
                # even more trouble in paradise
                self.logger.exception("start_main_loop: StandardError, %s, %s" % (self.activity.get_name(), info))
                self.run_event_loop = False
                self.run_main_loop = False
                error = True
                self.store_data = False
            except KeyboardInterrupt:
                try:
                    utils.load_music(os.path.join(CORESOUNDSDIR, 'hal.ogg')).play()
                    pygame.time.wait(10000)
                except:
                    pass
                self.logger.exception("KeyboardInterrupt stopping all threads")
                try:
                    self.activity.stop_timer()
                except:
                    pass
                self.logger.warning("Raising SystemExit to quit")
                raise SystemExit
                
        self.logger.debug("Break out of start_main_loop")
        if self.activity.get_name() != 'menu' or error:
            self.activity_game_end(store_db=self.store_data)
        else:
            self.ask_exit()# if this returns the user said no
            self.logger.debug("Restarting menu")
            self.activity.start()
            text = _("Current activity : %s") % self.activity.get_helptitle()
            if self.activitylabel:
                self.activitylabel.settext(text)
                self.activitylabel.display_sprite()
            self.run_main_loop = True
            self.start_main_loop()
            
    def start_pre_level_loop(self):
        # Start the pre_level method once and calls the eventloop.
        self.were_in_pre_level = False
        self.were_in_pre_level = True
        if self.activity.get_name() not in ('dltr', 'quiz_general') and self.DTmode:
            if not self.menubuttons_backup:
                self.dicebuttons.next_level(self.DTlevelcount)
            self.logger.debug("Calling dailytraining_pre_level with level: %s" % (self.DTlevelcount))
            self.run_event_loop = self.activity.dailytraining_pre_level(self.DTlevelcount)
        else:
            if not self.menubuttons_backup:
                self.dicebuttons.next_level(self.levelcount)
            self.logger.debug("Calling pre_level with level: %s" % (self.levelcount+1))
            self.run_event_loop = self.activity.pre_level(self.levelcount + 1)
        # enter activity level eventloop if run_event_loop == True
        if self.run_event_loop:
            self.logger.debug("Enter pre_level eventloop")
            self._main_loop()
            self.logger.debug("run_event_loop done")
        self.were_in_pre_level = False
        self.run_activity_loop = True
        self.run_event_loop = False 
        
        self.start_activity_loop()

    def start_activity_loop(self):
        # start stepping throught the activity levels
        while self.run_activity_loop:
            try:
                self.logger.debug("Start next_level")
                if self.DTmode and self.DTactivity.get_name() in ('dltr', 'quiz_general'):
                    levelcount = self.DTlevelcount
                else:
                    levelcount = self.levelcount
                self.start_next_level(store_db=True, nextlevel=levelcount)
            except utils.MyError, info:
                self.logger.error("Exception raised in the activity %s: %s" % (self.activity.get_name(), info))
                if self.activity.get_name() == 'menu':
                    self.logger.error('Exception occurred in the menu, quiting')
                    raise utils.SPError
                self.run_event_loop = False
                self.run_activity_loop = False
                self.run_main_loop = False
                self.store_data = False
                #self.activity_game_end()
            # block events from entering the queue.
            # Pro: cpu usage drops dramaticly.
            # Con: gui can't show fancy color changes in the widgets as the 
            #       mouse passes over them.
#            if self.activity.get_name() == 'menu':
#                # we need mousemotion for the tooltips
#                pygame.event.set_allowed([KEYDOWN, MOUSEBUTTONDOWN, MOUSEMOTION])
#            else:
#                # and here we block mousemotion to conserve cpu usage
#                pygame.event.set_blocked(MOUSEMOTION)
            else:
                self.run_event_loop = True
                # enter activity level eventloop
                self.logger.debug("Enter eventloop")
                self._main_loop()
                self.logger.debug("run_event_loop done")

    def _main_loop(self):
        self.logger.debug("Starting main_loop with framerate %s" % self.framerate)
        pygame.event.clear()
        self.myspritesactivated = None
        while self.run_event_loop:
            self.clock.tick(self.framerate)
            pygame.event.pump()
            events = pygame.event.get()
            for event in events:
                if event.type == KEYDOWN:
                    if event.key is K_ESCAPE or event.type is QUIT:
                        self.run_event_loop = False
                        raise EscapeKeyException
                    elif event.key == K_F1 and self.logger.isEnabledFor(logging.DEBUG) and not utils.WEAREPRODUCTION:
                        self.start_F1_screen()
                    # Only for debugging memoryleaks
                    #elif event.key == K_F9:
                    #    utils.memory_profile(h)
                self.actives_group.update(event) 
            if self.myspritesactivated:
                self.myspritesactivated = None
                continue
            # Call the activity loop.
            try:
                self.activity.loop(events)
                # prevent 'old' events too be reprocessed.
                # we now fixed this is findit and quizgeneral, for now just leave it
                # commented out
                #pygame.event.clear()
            except utils.StopGameException, info:
                self.logger.debug("_main_loop, StopGameException")
                # stop loops
                self.run_event_loop = False
                self.run_activity_loop = False
                self.activity_game_end(store_db=False)
            except StandardError, info:
                self.activity.stop_timer()
                self.logger.exception("%s raised an unhandled exception: %s traceback follows:" % \
                    (self.activity.get_name(), info))
                self.activity_info_dialog(_("There was an error in %s. Please check the logs and inform the developers") % self.activity.get_name())
                # make sure we have enabled menubuttons
                self.enable_menubuttons()
                # stop loops
                self.run_event_loop = False
                self.run_activity_loop = False
                self.activity_game_end(store_db=False)
                
    #### stuff for beta testers #####################################################
    def start_F1_screen(self):
        import SPDebugDialog
        org_screen = self.screen.convert()
        org_back = self.backgr.convert()
        self.lock.acquire()
        try:
            db = SPDebugDialog.Debugscreen(self.activity.get_name(), MILESTONE, self.screen)
            actives = SPGroup(self.screen, self.backgr)
            actives.add(db.get_actives())
            runloop = 1 
            while runloop:
                pygame.time.wait(100)
                pygame.event.pump()
                events = pygame.event.get()
                for event in events:
                    if event.type is KEYDOWN:
                        if event.key == K_ESCAPE:
                            runloop = 0
                    result = actives.update(event)
                    if result and result[0][1] == -2:
                        runloop = False
                    if result and result[0][1] == -3:
                        # git pull, restart myself
                        raise utils.RestartMeException
        except utils.MyError, info:
            self.activity_info_dialog("There was an error: %s" % info)
        self.screen.blit(org_screen, (0, 0))
        self.backgr.blit(org_back, (0, 0))
        pygame.display.update()
        self.lock.release()
    #################################################################################

if __name__ == '__main__':
    utils.set_locale()# TODO: add lang argument
    try:
        mcgui = MainCoreGui((800, 600))
    except SystemExit, status:
        if str(status) == '0':
            module_logger.info("clean exit")
        else:
            module_logger.info("not a clean exit")
    except Exception, status:
        module_logger.exception("unhandled exception in toplevel, traceback follows:")
            



# -*- coding: utf-8 -*-

# Copyright (c) 2007-2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPDataManager.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# TODO: what do we do when a error in dbase stuff occurs?

#create logger, logger was configured in SPLogging
import logging
module_logger = logging.getLogger("childsplay.SPDataManager")

import atexit, os, sys, datetime

# Don't do from 'sqlalchemy import *' as SQA has also 'logging' and 'types'
# modules. This is very bad coding practice but they claim to have good reasons
# for it. Those reasons suck of course but I don't have the time to discuss it
# with them. So I will just use practices *I* think are right and which I should
# have used to begin with and that's '*never* do from foo import *'.
# The braindead part of it all is that SQA use 'from sqlalchemy import *' in their
# docs and tutorials :-(
# None the less, SQA is a very good lib.
from SPConstants import ACTIVITYDATADIR
import SPHelpText
from utils import MyError, StopmeException
try:
    import sqlalchemy as sqla
    import sqlalchemy.orm as sqlorm
except ImportError:
    module_logger.exception("No sqlalchemy package found")
    raise MyError

# attempt to prevent sqlalchemy trhowing recursion limit error
sys.setrecursionlimit(2000) # 1000 is the default

from utils import set_locale

#import SPgdm

from SPDataManagerCreateDbase import DbaseMaker
DEBUG = False

DEMO_DT = [{'fortune': 0, 'target': 'demo', 'level': 2, 'group': 'Lange termijn', 'cycles': 1,'act_name': 'quiz_picture', 'order': 0}, \
           {'fortune': 0, 'target': 'demo', 'level': 2, 'group': 'Puzzels', 'cycles': 1,'act_name': 'electro_sp', 'order': 1}, \
           {'fortune': 0, 'target': 'demo', 'level': 2, 'group': 'Lange termijn', 'cycles': 1,'act_name': 'quiz_melody', 'order': 2},\
           ]
DEFAULT_DT = [{'fortune': 0, 'target': 'default', 'level': 2, 'group': 'Lange termijn', 'cycles': 1,'act_name': 'quiz_picture', 'order': 0},\
           {'fortune': 0, 'target': 'default', 'level': 2, 'group': 'Puzzels', 'cycles': 2,'act_name': 'electro_sp', 'order': 1}, \
           {'fortune': 0, 'target': 'default', 'level': 2, 'group': 'Lange termijn', 'cycles': 1,'act_name': 'quiz_math', 'order': 2},\
           {'fortune': 0, 'target': 'default', 'level': 2, 'group': 'Puzzels', 'cycles': 2,'act_name': 'numbers_sp', 'order': 3},\
           {'fortune': 0, 'target': 'default', 'level': 2, 'group': 'Lange termijn', 'cycles': 1,'act_name': 'quiz_sayings', 'order': 4},\
           {'fortune': 0, 'target': 'default', 'level': 2, 'group': 'Korte termijn', 'cycles': 2,'act_name': 'memory_sp', 'order': 5},\
           {'fortune': 0, 'target': 'default', 'level': 2, 'group': 'Lange termijn', 'cycles': 1,'act_name': 'quiz_picture', 'order': 6}, \
           {'fortune': 0, 'target': 'default', 'level': 2, 'group': 'Puzzels', 'cycles': 2,'act_name': 'findit_sp', 'order': 7}, \
           {'fortune': 0, 'target': 'default', 'level': 2, 'group': 'Lange termijn', 'cycles': 1,'act_name': 'quiz_melody', 'order': 8}
              ]
EASY_DT = [{'fortune': 0, 'target': 'Easy', 'level': 2, 'group': 'Lange termijn', 'cycles': 1, 'act_name': 'quiz_picture', 'order': 0},\
           {'fortune': 0, 'target': 'Easy', 'level': 2, 'group': 'Puzzels', 'cycles': 3, 'act_name': 'electro_sp', 'order': 1},\
           {'fortune': 0, 'target': 'Easy', 'level': 2, 'group': 'Lange termijn', 'cycles': 1, 'act_name': 'quiz_sayings', 'order': 2},\
           {'fortune': 0, 'target': 'Easy', 'level': 2, 'group': 'Puzzels', 'cycles': 3, 'act_name': 'puzzle', 'order': 3},\
           {'fortune': 0, 'target': 'Easy', 'level': 2, 'group': 'Lange termijn', 'cycles': 1, 'act_name': 'quiz_math', 'order': 4},\
           {'fortune': 0, 'target': 'Easy', 'level': 2, 'group': 'Lange termijn', 'cycles': 1, 'act_name': 'quiz_melody', 'order': 5},\
           ]

HARD_DT = [{'fortune': 0, 'target': 'Hard', 'level': 2, 'group': 'Lange termijn', 'cycles': 1, 'act_name': 'quiz_picture', 'order': 0},\
         {'fortune': 0, 'target': 'Hard', 'level': 3, 'group': 'Puzzels', 'cycles': 3, 'act_name': 'electro_sp', 'order': 1},\
         {'fortune': 0, 'target': 'Hard', 'level': 2, 'group': 'Lange termijn', 'cycles': 1, 'act_name': 'quiz_sayings', 'order': 2},\
         {'fortune': 0, 'target': 'Hard', 'level': 3, 'group': 'Korte termijn', 'cycles': 3,'act_name': 'memory_sp', 'order': 3}, \
         {'fortune': 0, 'target': 'Hard', 'level': 2, 'group': 'Lange termijn', 'cycles': 1,'act_name': 'quiz_history', 'order': 4}, \
         {'fortune': 0, 'target': 'Hard', 'level': 2, 'group': 'Korte termijn', 'cycles': 3, 'act_name': 'soundmemory', 'order': 5},\
         {'fortune': 0, 'target': 'Hard', 'level': 2, 'group': 'Lange termijn', 'cycles': 1, 'act_name': 'quiz_math', 'order': 6},\
         {'fortune': 0, 'target': 'Hard', 'level': 2, 'group': 'Puzzels', 'cycles': 3, 'act_name': 'numbers_sp', 'order': 7},\
         {'fortune': 0, 'target': 'Hard', 'level': 3, 'group': 'Lange termijn', 'cycles': 1, 'act_name': 'quiz_math', 'order': 8},\
         {'fortune': 0, 'target': 'Hard', 'level': 2, 'group': 'Puzzels', 'cycles': 3, 'act_name': 'fourrow', 'order': 9},\
         {'fortune': 0, 'target': 'Hard', 'level': 2, 'group': 'Lange termijn', 'cycles': 1, 'act_name': 'quiz_melody', 'order': 10}\
         ]

class DataManager:
    """Class that handles all users data related stuff except the collecting that
    should be done by the activity."""
    def __init__(self, spgoodies, dbm):
        self.logger = logging.getLogger("childsplay.SPDataManager.DataManager")
        self.logger.debug("Starting")
        self.SPG = spgoodies
        self.cmd_options = self.SPG._cmd_options
        self.current_user = self.cmd_options.user
        self.current_user_id = None
        self.COPxml = None# controlpanel stuff
        atexit.register(self._cleanup)

        self.content_engine, self.user_engine = dbm.get_engines()
        self.metadata_contentdb, self.metadata_usersdb = dbm.get_metadatas()
        self.all_orms = dbm.get_all_orms()
        self.orms_content_db, self.orms_userdb = dbm.get_orms()
        self.UserSession = sqlorm.sessionmaker(bind=self.user_engine)
        self.ContentSession = sqlorm.sessionmaker(bind=self.content_engine)
        # query which language we should use.
        orm, session = self.get_orm('spconf', 'user')
        row = session.query(orm).filter_by(activity_name = 'language_select')\
                                    .filter_by(key = 'locale').first()
        if not row:
            language = self.cmd_options.lang
            if not language:
                language = self.cmd_options.default_language
            row = orm(activity_name='language_select', key='locale', value=language, comment='locale used by the core')
            session.add(row)
            row = orm(activity_name='language_select', key='lang', value=language[:2], comment='language code used by the core')
            session.add(row)
            session.commit()
            session.close()
            language = set_locale(language)
        elif not self.cmd_options.lang:
            language = set_locale(row.value)
        else:
            language = self.cmd_options.lang
            if not language:
                language = self.cmd_options.default_language
            language = set_locale(language)    
        self.language = language
        self.SPG.localesetting = language
        self._check_tables_uptodate()
        # query to get all availabe cids, used to check served_content
        orm, session = self.get_orm('game_available_content', 'content')
        query = session.query(orm)
        self.all_ids = [result.CID for result in query.all()]
        session.close()
        
        if self.cmd_options.no_login:
            self.current_user = 'SPUser'
            self._start_gdm_greeter()
        elif self.cmd_options.user:
            self.current_user = self.cmd_options.user
            self._start_gdm_greeter()
        elif self.SPG.get_theme() == 'braintrainer':
            self.WeAreBTP = True
            self._start_btp_screen()
        else:
            self.WeAreBTP = False
            # we don't have a working login screen yet
            self.current_user='SPUser'
            self._start_gdm_greeter()
    
    def reset(self):
        self.UserSession.close_all()
        self.ContentSession.close_all()
        try:
            self.user_engine.dispose()
            self.content_engine.dispose()
        except:
            pass
    
    def _get_language(self):
        return self.language
    
    def _check_tables_uptodate(self):
        self.logger.debug("_check_tables_uptodate")
        reload(SPHelpText)
        modules = [x for x in os.listdir(ACTIVITYDATADIR) if '.py' in x and not '.pyc' in x]
        # check that all the activities are present in the activity_options table
        orm, session = self.get_orm('activity_options', 'user')
        if orm == None:
            self.logger.error("No activity_options ORM found, dbase corrupt")
            raise MyError, "No activity_options ORM found, dbase corrupt"
        for m in modules:
            m = m[:-3]
            query = session.query(orm)
            query = query.filter_by(activity = m)
            result = query.first()
            if not result:
                # Not found activity name, set activity name with default values
                session.add(orm(m))
        session.commit()
        session.close()
                                        
        orm, session = self.get_orm('group_names', 'user')
        
        # make user demo    
        orm, session = self.get_orm('users', 'user')
        result = session.query(orm).filter_by(login_name = 'Demo').first()
        if not result:
            session.query(orm).filter_by(user_id = 1).delete()
            neworm = orm()
            neworm.user_id = 1
            neworm.first_name = 'Demo'
            neworm.last_name = ''
            neworm.login_name = 'Demo'
            neworm.audio = 50
            neworm.usersgroup = 0
            neworm.dt_target = 'demo'
            session.add(neworm)
        session.commit()
        session.close()
        # check for mandatory DT sequences
        orm, session = self.get_orm('dt_sequence', 'user')
        query = session.query(orm).filter_by(target = 'demo').all()
        if len(query) != len(DEMO_DT):
            self.logger.info("demo dt target differs from hardcoded sequence, replacing it")
            session.query(orm).filter(orm.target == 'demo').delete()
            session.commit()
            for row in DEMO_DT:
                session.add(orm(**row))
        query = session.query(orm).filter_by(target = 'default').all()
        if not query:
            self.logger.info("default dt target missing, adding a hardcoded sequence.")
            session.query(orm).filter(orm.target == 'default').delete()
            session.commit()
            for row in DEFAULT_DT:
                session.add(orm(**row))
        session.commit()
        session.close()
        val = self._get_rcrow('SPDatamanager', 'set_extra_dt_sequences')
        if not val or val != 'yes':
            # we also set two DT sequences once, user can remove them
            orm, session = self.get_orm('dt_sequence', 'user')
            query = session.query(orm).filter_by(target = 'Easy').all()
            if not query:
                self.logger.info("First time Easy dt target missing, adding a hardcoded sequence.")
                session.query(orm).filter(orm.target == 'Easy').delete()
                session.commit()
                for row in EASY_DT:
                    session.add(orm(**row))
            query = session.query(orm).filter_by(target = 'Hard').all()
            if not query:
                self.logger.info("First time Hard dt target missing, adding a hardcoded sequence.")
                session.query(orm).filter(orm.target == 'Hard').delete()
                session.commit()
                for row in HARD_DT:
                    session.add(orm(**row))
            session.commit()
            session.close()
            self._set_rcrow('SPDatamanager', 'set_extra_dt_sequences', 'yes', 'flag to check if we already have set the extra dt sequences')
        
    def _cleanup(self):
        """atexit function"""
        # Nothing to see here, please move on.
        self.reset()
    
    def _start_btp_screen(self):
        """Starts a login screen for the braintrainer plus.
        Beaware that this only works on a BTP system as the login and
        control panel is a proprietary piece of code and it's not included
        in the free versions."""
        sys.path.insert(0, './controlpanel_lgpl')
        import Start_screen as Ss #@UnresolvedImport
        self.SPG.dm = self
        ss = Ss.Controller(self.SPG, fullscr=self.cmd_options.fullscreen)
        result = ss.get_result()
        if result[0] == 'user':
            self.current_user = result[1]
            self._start_gdm_greeter()
        elif result[0] == 'quit':
            raise StopmeException, 0
        elif result[0] == 'controlpanel':
            self.COPxml = result[1]
            
    def are_we_cop(self):
        return self.COPxml
        
    def _start_gdm_greeter(self):
        """Will start login screen and stores the login name in the db"""
        self.current_user = 'Demo'
        if not self.current_user:
            g = SPgdm.SPGreeter(self.cmd_options, \
                theme=self.cmd_options.theme, \
                vtkb=self.SPG.get_virtual_keyboard(), \
                fullscr=self.cmd_options.fullscreen)# returns when user hits login button
            username = g.get_loginname()
        else:
            self.logger.debug("Username %s passed as cmdline option, no login screen" % self.current_user)
            username = self.current_user
        self.logger.debug("Got login: %s" % username)
        if not username:
            # we always must run under a user name so we use default
            username = self.cmd_options.user
            self.logger.debug("No login, setting username to default: %s" % username)
        
        # Now that we have a name we first check if it already exists
        # get the users table
        orm, session = self.get_orm('users', 'user')
        query = session.query(orm)
        query = query.filter_by(login_name = username)
        result = query.first()
        if result:
            self.logger.debug("found existing username: %s" % result.login_name)
        else:
            # insert just user_name, NULL for others, the user_id will be generated
            session.add(orm(login_name=username, first_name=username, usersgroup='SPusers'))
            self.logger.debug("inserted %s" % username)
            session.commit()
            query = session.query(orm)
            query = query.filter_by(login_name = username)
            result = query.first()
            session.close()
            # we must also check if the SPusers group exists.
            orm, session = self.get_orm('group_names', 'user')
            rows = [row for row in session.query(orm).order_by(orm.group_name).all()]
            if not rows:
                # we set a first group
                neworm = orm()
                neworm.group_name = 'SP Group'
                session.add(neworm)
                session.commit()
            session.close()
        self.logger.debug("%s has user id %s" % (username, result.user_id))
        self.current_user_id = result.user_id
        self.current_user = username

    def get_username(self):
        """Returns the current user or None if in anonymousmode"""
        self.logger.debug("get_username returns:%s" % self.current_user)
        if not self.current_user:
            return ''
        return self.current_user
        
    def get_user_id(self):
        return self.current_user_id
        
    def get_user_id_by_loginname(self, username):
        """Returns the user_id.
        @username must be the users login name"""
        orm, session = self.get_orm('users', 'user')
        query = session.query(orm)
        query = query.filter_by(login_name = username)
        result = query.first()
        if not result:
            self.logger.warning("No user %s found, expect more trouble :-(" % username)
        else:
            return result.user_id
            
    def get_user_dbrow_by_loginname(self, username):
        """Returns the user_id.
        @username must be the users login name"""
        orm, session = self.get_orm('users', 'user')
        query = session.query(orm)
        query = query.filter_by(login_name = username)
        result = query.first()
        if not result:
            self.logger.warning("No user %s found, expect more trouble :-(" % username)
            return 
        else:
            return result
    
    def get_table_names(self):
        """Returns a list with the names (strings) of the SQL tables currently in use."""
        tl = self.metadata_usersdb.tables.keys()
        return tl
    
    def get_orm(self, tablename, dbase):
        try:
            t = self.all_orms[tablename]
        except KeyError:
            self.logger.warning("get_orm No such table: %s" % tablename)
            return None,None
        else:
            if dbase == 'user':
                self.user_engine.dispose()
                return (t, self.UserSession())
            elif dbase == 'content':
                self.content_engine.dispose()
                return (t, self.ContentSession())
            else:
                self.logger.warning("no such dbase: %s" % t)
                return None, None
            
    def get_served_content_orm(self):
        return self.get_orm('served_content', 'user')

    def get_table_data_userdb(self, table):
        orm, session = self.get_orm(table, 'user')
        query = session.query(orm)
        return query.all()
    
    def get_mu_sigma(self, name):
        orm, session = self.get_orm('activity_options', 'user')
        query = session.query(orm)
        query = query.filter_by(activity = name)
        result = query.first()
        if not result:
            self.logger.warning("Not found mu and sigma for %s, expect more trouble :-(" % name)
            return 
        return (result.mu, result.sigma)    

    def get_served_content_mapper(self):
        orm, session = self.get_orm('served_content', 'user')
        mclass = ServedMapper(orm, session, self.current_user_id, self.current_user)
        return mclass
        
    def get_mapper(self, activity, dbase='user'):
        self.logger.debug("get_mapper called with activity:%s" % activity)
        #self.metadata_usersdb.bind.echo = True
        if not activity:
            self.logger.debug("anonymous or no activity, returning bogus")
            return BogusMapper()
        try:
            orm, session = self.get_orm(activity, dbase)
            mclass = RowMapper(orm, session, self.current_user_id, self.current_user)
        except (KeyError, TypeError):
            self.logger.warning("Failed to get mapper or activity doesn't have a dbase table : %s, returning bogus mapper" % activity)
            return BogusMapper()
        else:
            return mclass

    # Used by multiple acts through spgoodies
    def _check_already_served(self, rows, game_theme, minimum=10, all_ids=None):
        """Returns the rows with the ones that are served removed.
        When not enough 'free' rows are left it resets all the count_served fields
        and return the complete rows list.
        all_ids is a list with with possible ids to check against served ids."""
        self.logger.debug("_check_already_served called: %s rows offered" % len(rows))
        if not all_ids:
            all_ids = self.all_ids
        orm, session = self.get_served_content_orm()
        query = session.query(orm)
        query = query.filter_by(user_id = self.current_user_id)
        query = query.filter(orm.game_theme_id.in_(game_theme))
        query = query.filter(orm.count_served > 0)
        allrows = []
        served_ids = []
        for row in query.all():
            allrows.append(row)
            served_ids.append(row.CID)
        self.logger.debug("already served rows: %s" % len(served_ids))
        notserved = set(all_ids).difference(served_ids)
        self.logger.debug("found %s not served cids" % len(notserved))
        if len(notserved) < minimum:
            # Not enough unserved rows
            # first we set all the count_served back to 0
            query = session.query(orm).filter_by(user_id = self.current_user_id)
            query = query.filter(orm.game_theme_id.in_(game_theme))
            query.update({orm.count_served: 0}, synchronize_session=False)
            session.commit()
            session.close()
            # we now return all rows as there are now considered not yet served.
            self.logger.debug("Resetting served count and returning %s original rows" % len(rows))
            return rows
        else:
            # We must filter the rows by removing nonfree ones
            session.close()
            rows = [row for row in rows if row.CID in notserved]
            self.logger.debug("returning %s rows" % len(rows))
            return rows

    def _set_rcrow(self, actname, key, value, comment):
        orm, session = self.get_orm('spconf', 'user')
        query = session.query(orm).filter_by(activity_name = actname).filter_by(key = key).all()
        for row in query:
            session.delete(row)
        row = orm(activity_name=actname, key=key, value=value, comment=comment)
        session.add(row)
        session.commit()
        session.close()

    def _get_rcrow(self, actname, key):
        val = None
        orm, session = self.get_orm('spconf', 'user')
        query = session.query(orm).filter_by(activity_name = actname).filter_by(key = key).first()
        if query:
            val = query.value
        session.commit()
        session.close()
        return val
     
    def _update_rcrow(self, actname, key, val):
        orm, session = self.get_orm('spconf', 'user')
        query = session.query(orm).filter_by(activity_name = actname).filter_by(key = key).first()
        if query:
            comm = query.comment
        session.commit()
        session.close()
        self._set_rcrow(actname, key, val, comm)
        
class RowMapper:
    """DB object used by the core and activity to store data in the dbase
    table and row beloging to the current activity.
    Don't use this class directly, use the DataManagers get_mapper method."""
    def __init__(self, orm, session, user_id=None, current_user=''):
        self.logger = logging.getLogger("childsplay.SPDataManager.RowMapper")
        self.currentuser = current_user
        self.user_id = user_id
        self.orm = orm
        self.session = session
        self.coldata = {}
            
    def insert(self, col, data):
        """collects all the data which should go into a row.
        You must call 'commit' to actually store it into the dbase."""
        self.logger.debug("insert in %s: %s" % (col, data))
        self.coldata[col] = data
            
    def update(self, rowdata):
        """insert a row in to the current table.
        @rowdata must be a dictionary with column keys and data values.
        You must call 'commit' to actually store it into the dbase."""
        self.coldata.update(rowdata)
    
    def commit(self):
        """Flush dbase data to disk.
        Returns None on success and True on faillure."""
        if not self.orm:
            self.logger.info("No orm, nothing to store")
            return

        self.logger.debug("orm %s commit data to dbase" % self.orm._name)
        if hasattr(self.orm, 'user_id'):
            self.insert('user_id', self.user_id)
        self.logger.debug("raw row data:%s" % self.coldata)
        self.session.add(self.orm(**self.coldata))
        if not self.session:
            return
        self.session.commit()
        self.session.close()

    def _get_level_data(self, levelnum=1):
        """Used by maincore"""
        query = self.session.query(self.orm)
        query.filter_by(level = levelnum)
        query.filter_by(user_id = self.user_id)
        return query.all()
    
    def _get_start_time(self):
        """Used by the maincore"""
        if self.coldata.has_key('start_time'):
            return self.coldata['start_time']
        
    def _get_end_time(self):
        """Used by the maincore"""
        if self.coldata.has_key('end_time'):
            return self.coldata['end_time']
    
    def get_orm(self):
        return self.orm
    def get_session(self):
        return self.session
    def close(self):
        if not self.session:
            return
        self.session.close()

class ServedMapper:
    """DB object for the served_content table in the users db.
    Used by the core and activity to store data in the dbase
    table and row beloging to the current activity.
    Don't use this class directly, use the DataManagers get_mapper method."""
    def __init__(self, orm, session, user_id=None, current_user=''):
        self.logger = logging.getLogger("childsplay.SPDataManager.ServedMapper")
        self.currentuser = current_user
        self.user_id = user_id
        self.orm = orm
        self.session = session
        self.coldata = {}
          
    def insert(self, cid, gtheme):
        """collects all the data which should go into a row.
        You must call 'commit' to actually store it into the dbase."""
        self.logger.debug("insert cid:%s game_theme_id:%s" % (cid, gtheme))
        svc = self.orm(user_id=self.user_id, CID=cid,\
                       game_theme_id=gtheme, \
                       module='', start_time=datetime.datetime.now(), \
                        count_served=1)
        self.session.add(svc)
        
    def commit(self):
        if not self.session:
            return
        self.logger.debug("commiting session")
        self.session.commit()
        self.session.close()
        
    def close(self):
        if not self.session:
            return
        self.session.close()
    
class BogusMapper:
    """Bogus mapper class used when we are in anonymousmode"""
    def __init__(self):
        self._name = 'Bogusmapper'
    def __str__(self):
        return "BogusMapper"
    def __repr__(self):
        return "BogusMapper"
    def insert(self, col, data):
        pass
    def insert_row(self, rowdata):
        pass
    def update(self, rowdata):
        pass
    def commit(self):
        pass
    def close(self):
        pass
    def get_table_column_names(self):
        pass
    def get_table_data(self):
        pass  
    def delete_row(self, row_id):
        pass
    def get_table_selection(self, args):
        pass
    def _get_start_time(self):
        return "2000-01-01_00:00:00"
    def _get_end_time(self):
        return "2000-01-01_00:00:00"
    def _get_level_data(self, level=1):
        return None




# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPSoundServer.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Simple threaded soundserver with minimal features to make it as bullutproof
# as possible. We must never leave a stale soundserver thread running after
# a crash or whatever. It's nearly impossible for BT users to kill threads.

import os,sys,time
import utils
import logging
from threading import Thread,enumerate
import atexit
import pygame

class SoundServer:
    """ Threaded soundserver which plays the files placed on the queue.
    It checks for the existence of the file and when it doesn't exists it will
    be removed from the queue and a error message will be placed in the logs.
     """         
    server_allready_running = False
    def __init__(self, sleep=0.2):
        """ This will start the server, the sleep value is the time the server
        will sleep between polling the message queue"""
        self.logger = logging.getLogger("childsplay.SPSoundServer")
        if self.server_allready_running:
            self.logger.debug("Soundserver allready running. You should first stop the server before you start a new one.")
            return
        # These are the attributes which shared between the threads
        self.sleep = sleep
        self.queue = []
        self.workingserver = True
        self.serverstatus = True
        self.serverping = False
        self.stopplaying = False
        self.start_server()
                        
    def __del__(self):
        try:
            self.stop_server(0)
        except:
            pass

    def start_server(self):
        if not self.server_allready_running:
            self.queue = []
            self.workingserver = True
            self.logger.debug("(re)starting soundserver")
            try:
                self.thread = Thread(target=self._start_server)
                self.thread.start()
                time.sleep(0.1)
            except Exception,info:
                self.logger.exception("SoundServer startup problem")
                self.serverstatus = False
                self.server_allready_running = False
            else:
                self.logger.debug("SoundServer running: %s" % self.thread)
                self.server_allready_running = True

    def stop_server(self,s=1):
        """ Stops a running server, after one second(default). 
        The delay is for stuff to finish, you can use stop_server(0) to stop now."""
        if not hasattr(self,"thread") or self.workingserver == 0:
            return
        self.logger.debug("stop_server called with delay: %s" % s)
        time.sleep(s)
        self.workingserver = False
        time.sleep(0.2)# give the thread the chance to terminate itself
        # Or in case it's playing something we just kill it :-/
        pygame.mixer.music.stop()
        self.logger.debug("result of stop_server (remaining threads):%s" % enumerate())
        self.server_allready_running = False
        self.serverstatus = False

    def play(self, file):
        """Creates a SDL music object from the file and places the file
        on the stack for the server to play.
        It returns True when the server isn't running or the file doesn't exists.
        """
        self.stopplaying = False
        if not self.server_allready_running:
            self.logger.error("sound server not running")
            return True
        if file != "PING" and not os.path.exists(file):
            self.logger.error("SoundServer: %s doesn't exists" % file)
            self.stop()
            return True
        #self.logger.debug("placed %s on the queue" % file)
        self.queue.insert(0, file)# The server checks the stack for messages
        return
    
    def clear_queue(self):
        """Clears the queue, useful when you want to interrupt the playing and
        play a new file."""
        self.queue = []
    
    def stop(self):
        """Stops the music but leaves the server running."""
        pygame.mixer.music.stop()
        self.stopplaying = True
        #self.queue = []
    
    def ping_server(self):
        """ Use this to test if the server is running.
        This actually sends a message to the server and the server signals
        back, this is done to check the message queue aswell.
        Be aware that this call pauses for half a second the current thread.
        It return True when the server is running and False when it's not."""
        self.serverping = False
        self.queue.insert(0,("PING", utils.NoneSound))
        time.sleep(0.5)
        self.logger.debug("result of ping_server:%s" % self.serverping)
        if self.serverping: 
            return True
        else: 
            return False

    def _start_server(self):
        """ This runs in a seperate thread and plays the sound object from the queue
        in a seperate thread also. Don't call this method, it is called from the class 
        constructor."""
        while self.workingserver:
            try:
                msg = ''
                try:
                    if self.queue:
                        msg = self.queue.pop()
                        #self.logger.debug("message from stack: %s" % msg)
                except IndexError:
                    pass
                else:
                    if msg:
                        if msg == "PING":
                            self.logger.debug("server pinged: running")
                            self.serverping = True                    
                        else:
                            while pygame.mixer.music.get_busy():
                                if self.stopplaying:
                                    self.logger.debug("server stopplaying set to True, stop playing %s" % msg)
                                    #self.queue = []
                                    self.stopplaying = False
                                    pygame.mixer.music.stop()
                                    break
                                pygame.time.wait(200)# prevent cpu hogging
                            else:
                                #self.logger.debug("playing: %s" % msg)
                                pygame.mixer.music.load(msg)
                                pygame.mixer.music.play()
                        
                time.sleep(self.sleep)
            except:
                self.logger.exception("Troubles in server thread, stopping server.")
                pygame.mixer.music.stop()
                self.stop_server(0)
        try:
            pygame.mixer.music.stop()
        except:
            pass
        self.logger.debug("Server stopped")
                

if __name__ == '__main__':
    
    import __builtin__
    __builtin__.__dict__['_'] = lambda x:x
    
    import SPLogging
    SPLogging.set_level('debug')
    SPLogging.start()
        
    import pygame
    from pygame.constants import *
    pygame.init()    
    scr = pygame.display.set_mode((100, 100))
    
    ss = SoundServer()
    ss.play("test.ogg")
    ss.ping_server()
    ss.play("test.ogg")
    
    runloop = 1 
    while runloop:
        pygame.time.wait(500)
        pygame.event.pump()
        events = pygame.event.get()
        for event in events:
            if event.type is KEYDOWN:
                if event.key == K_ESCAPE:
                    print "escape hit, stopping main"
                    runloop = 0
        print "main doing stuff"
    
    ss.stop_server(0)
    raw_input("hit any key to quit")




# -*- coding: utf-8 -*-

# Copyright (c) 2013 Stas Zytkiewicz stas.zytkiewicz@gmail.com
# Copyright (c) 2013 Ana Lopes anajoaolopes@gmail.com
# Copyright (c) 2013 Helder Costa h24costa@gmail.com
# Copyright (c) 2013 Rui Bento rui.m.bento@gmail.com
#
#           BlockBreaker.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation. A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("schoolsplay.BlockBreaker.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("schoolsplay.BlockBreaker")

# standard modules you probably need
import os,sys, random, math

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils

class Activity:
    level = 0
    angle_var = 30 #Variation angle when the ball hit the base
    pontos = 0
    nv_comp = 0
    pos_x = 0
    pos_y = 0

    # Define the colors we will use in RGB format
    BLUE1 =  (  13,   52, 169)
    BLUE2 =  (  44,   83, 203)
    BLUE3 =  (  133,   161, 242)
    GREEN1 = ( 12, 114, 15)
    GREEN2 = (  50, 191,   55)
    GREEN3 = (  134, 236,   138)
    ORANGE1 = (  198, 59,   4)
    ORANGE2 = (  239, 105,   9)
    ORANGE3 = (  245, 160,   99)
    GRAY1 = (  115, 115,   115)
    GRAY2 = (  175, 175,   175)

    # Player Base
    player = pygame.Rect(350, 570, 90, 10)

    #Blocks
    blocks = []

    # set up movement variables
    moveLeft = False
    moveRight = False

    # Base speed
    BaseSpeed = 5

    # Ball
    Ball_x = 400
    Ball_y = 560
    Ball_angle = random.randint(45, 135)
    Ball_speed = 5
    
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("schoolsplay.BlockBreaker.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','BlockbreakerData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'BlockBreaker.rc'))
        self.logger.debug("rchash: " + str(self.rchash))
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
                

    
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.refresh()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Blockbreaker")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "BlockBreaker"
    
    def get_help(self):
        """Mandatory methods"""
        text = ["Objective:",
        " ",
        "Try to complete, first, a row of 3 matching symbols,",
        "horizontally, vertically or diagonally.",
        " ",
        "Tip:",
        " ",
        "If your opponent has two symbols on the same line,",
        "you must put your symbol on that line to block him.",
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")
    
    def start(self):
        """Mandatory method."""
        #Define nivel
        pass
   
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6
    
    def pre_level(self,level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        self.logger.debug("pre_level called with: %s" % level)
        pass
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("nextlevel called with: %s" % level)
        # Your top blit position, this depends on the menubar position 
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        self.level = level
        self.next_exercise()

        return True
    
    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        pass
        
    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        pass
        return True
        
        
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass

    def next_exercise(self):

        self.Ball_x = 400
        self.Ball_y = 560
        self.Ball_angle = random.randint(45, 135)

        self.Ball_speed = 5 + self.level * 1
        self.BaseSpeed  = 5 + self.level * 1

        self.player.left = 400 - self.player.width / 2

        # remove all elements from array
        for block in self.blocks[:]:
            self.blocks.remove(block)

        if self.level == 1:
            for i in range(8):
                self.blocks.append({'block':pygame.Rect(22 + i * 96, 120, 86, 30), 'color':self.GREEN1})
            for i in range(7):
                self.blocks.append({'block':pygame.Rect(65 + i * 96, 160, 86, 30), 'color':self.GREEN2})
            for i in range(8):
                self.blocks.append({'block':pygame.Rect(22 + i * 96, 200, 86, 30), 'color':self.GREEN3})

        if self.level == 2:
            for i in range(11):
                self.blocks.append({'block':pygame.Rect(18 + i * 70, 136, 64, 25), 'color':self.ORANGE1})
            for i in range(10):
                self.blocks.append({'block':pygame.Rect(50 + i * 70, 168, 64, 25), 'color':self.ORANGE2})
            for i in range(11):
                self.blocks.append({'block':pygame.Rect(18 + i * 70, 200, 64, 25), 'color':self.ORANGE3})

        if self.level == 3:
            for i in range(15):
                self.blocks.append({'block':pygame.Rect(22 + i * 51, 125, 45, 20), 'color':self.BLUE1})
            for i in range(14):
                self.blocks.append({'block':pygame.Rect(44 + i * 51, 150, 45, 20), 'color':self.BLUE2})
            for i in range(15):
                self.blocks.append({'block':pygame.Rect(22 + i * 51, 175, 45, 20), 'color':self.BLUE3})
            for i in range(14):
                self.blocks.append({'block':pygame.Rect(44 + i * 51, 200, 45, 20), 'color':WHITE})

        if self.level == 4:
            for i in range(5):
                self.blocks.append({'block':pygame.Rect(277 + i * 51, 150, 45, 20), 'color':self.BLUE1})
            for i in range(9):
                self.blocks.append({'block':pygame.Rect(175 + i * 51, 175, 45, 20), 'color':self.ORANGE2})
            for i in range(13):
                self.blocks.append({'block':pygame.Rect(73 + i * 51, 200, 45, 20), 'color':self.GREEN3})
            for i in range(9):
                self.blocks.append({'block':pygame.Rect(175 + i * 51, 225, 45, 20), 'color':self.ORANGE2})
            for i in range(5):
                self.blocks.append({'block':pygame.Rect(277 + i * 51, 250, 45, 20), 'color':self.BLUE1})

        if self.level == 5:
            for i in range(15):
                self.blocks.append({'block':pygame.Rect(22 + i * 51, 125, 45, 20), 'color':self.GRAY1})
            for i in range(14):
                self.blocks.append({'block':pygame.Rect(44 + i * 51, 150, 45, 20), 'color':self.GREEN1})
            for i in range(15):
                self.blocks.append({'block':pygame.Rect(22 + i * 51, 175, 45, 20), 'color':self.GREEN2})
            for i in range(14):
                self.blocks.append({'block':pygame.Rect(44 + i * 51, 200, 45, 20), 'color':self.GRAY1})

        if self.level == 6:
            for i in range(3):
                self.blocks.append({'block':pygame.Rect(22 + i * 51, 150, 45, 20), 'color':self.GRAY1})
            for i in range(3):
                self.blocks.append({'block':pygame.Rect(628 + i * 51, 150, 45, 20), 'color':self.GRAY1})
            for i in range(5):
                self.blocks.append({'block':pygame.Rect(44 + i * 51, 175, 45, 20), 'color':self.ORANGE2})
            for i in range(5):
                self.blocks.append({'block':pygame.Rect(504 + i * 51, 175, 45, 20), 'color':self.ORANGE2})
            for i in range(7):
                self.blocks.append({'block':pygame.Rect(22 + i * 51, 200, 45, 20), 'color':self.GRAY1})
            for i in range(7):
                self.blocks.append({'block':pygame.Rect(424 + i * 51, 200, 45, 20), 'color':self.GRAY1})                
            for i in range(5):
                self.blocks.append({'block':pygame.Rect(44 + i * 51, 225, 45, 20), 'color':self.ORANGE2})
            for i in range(5):
                self.blocks.append({'block':pygame.Rect(504 + i * 51, 225, 45, 20), 'color':self.ORANGE2})                
            for i in range(3):
                self.blocks.append({'block':pygame.Rect(22 + i * 51, 250, 45, 20), 'color':self.GRAY1})
            for i in range(3):
                self.blocks.append({'block':pygame.Rect(628 + i * 51, 250, 45, 20), 'color':self.GRAY1})


    def GameArea(self):
        # move the Player Base
        if self.moveLeft:
            self.player.left -= self.BaseSpeed
            if self.player.left < 10:
                self.player.left = 10
        if self.moveRight:
            self.player.right += self.BaseSpeed
            if self.player.right > 790:
                self.player.right = 790

        # move the Ball on x axis
        self.Ball_x = self.Ball_x + self.Ball_speed * math.cos( self.Ball_angle * math.pi / 180)

        # check if the ball hit left side
        if self.Ball_x <= 25 + self.Ball_speed:
            if self.Ball_angle < 180:
                self.Ball_angle = 180 - self.Ball_angle
            else:
                self.Ball_angle = 270 + 270 - self.Ball_angle

        # check if the ball hit right side
        if self.Ball_x >= 775 - self.Ball_speed:
            if self.Ball_angle < 90:
                self.Ball_angle = 180 - self.Ball_angle
            else:
                self.Ball_angle = 270 + 270 - self.Ball_angle

        # move the Ball on y axis
        self.Ball_y = self.Ball_y - self.Ball_speed * math.sin( self.Ball_angle * math.pi / 180)

        # check if the ball hit the top the screen
        if self.Ball_y <= 110 + self.Ball_speed:
            self.Ball_angle = 360 - self.Ball_angle

        # check if the ball hit the bottom of the screen
        if self.Ball_y >= 585 - self.Ball_speed:
            self.player.left = 400 - self.player.width / 2
            self.Ball_x = 400
            self.Ball_y = 560
            self.Ball_angle = random.randint(45, 135)
            self.moveLeft = False
            self.moveRight = False

        # check if the ball hit the player base
        if self.Ball_angle > 180:
            if self.Ball_x + 10 > self.player.left and self.Ball_x - 10 < self.player.right and self.Ball_y + 10 > self.player.top and self.Ball_y -10 < self.player.bottom:
                for i in range(0,100,10):

                    aux_x = self.Ball_x + 10 * math.cos( (270 + i) * math.pi / 180)
                    aux_y = self.Ball_y - 10 * math.sin( (270 + i) * math.pi / 180)

                    if self.player.collidepoint(aux_x, aux_y): 
                        if aux_y >= self.player.top and aux_y <= self.player.top + self.Ball_speed:
                            self.Ball_y = 560
                            self.Ball_angle = 360 - self.Ball_angle - ((self.Ball_x - (self.player.left + self.player.width / 2)) * 100 / ( self.player.width / 2 ) * self.angle_var /100)
                            break    
                        else:
                            self.Ball_angle = 270 + 270 - self.Ball_angle
                            self.player.left += self.Ball_speed
                            break

                    aux_x = self.Ball_x + 10 * math.cos( (270 - i) * math.pi / 180)
                
                    if self.player.collidepoint(aux_x, aux_y):
                        if aux_y >= self.player.top and aux_y <= self.player.top + self.Ball_speed:
                            self.Ball_y = 560
                            self.Ball_angle = 360 - self.Ball_angle - ((self.Ball_x - (self.player.left + self.player.width / 2)) * 100 / ( self.player.width / 2 ) * self.angle_var /100)
                            break    
                        else:
                            self.Ball_angle = 270 + 270 - self.Ball_angle
                            self.player.right -= self.Ball_speed
                            break

        # check if the ball hit the blocks
        if self.Ball_y < 250:
            for block in self.blocks[:]:
                for i in range(0, 360, 15):
                    aux_x = self.Ball_x + 10 * math.cos(i * math.pi / 180)
                    aux_y = self.Ball_y - 10 * math.sin(i * math.pi / 180)
                    if block['block'].collidepoint(aux_x, aux_y):
                        self.pontos += 1
                        if aux_x >= block['block'].left and aux_x <= block['block'].left + self.Ball_speed:
                            if block['color'] == self.GRAY1:
                                block['color'] = self.GRAY2
                            else:
                                self.blocks.remove(block)
                            if self.Ball_angle < 90:
                                self.Ball_angle = 180 - self.Ball_angle
                            else:
                                self.Ball_angle = 270 + 270 - self.Ball_angle
                            break
                        if aux_x <= block['block'].right and aux_x >= block['block'].right - self.Ball_speed:
                            if block['color'] == self.GRAY1:
                                block['color'] = self.GRAY2
                            else:
                                self.blocks.remove(block)
                            if self.Ball_angle < 180:
                                self.Ball_angle = 180 - self.Ball_angle
                            else:
                                self.Ball_angle = 270 + 270 - self.Ball_angle
                            break
                        if aux_y >= block['block'].top and aux_y <= block['block'].top + self.Ball_speed:
                            if block['color'] == self.GRAY1:
                                block['color'] = self.GRAY2
                            else:
                                self.blocks.remove(block)
                            self.Ball_angle = 360 - self.Ball_angle
                            break
                        if aux_y <= block['block'].bottom and aux_y >= block['block'].bottom - self.Ball_speed:
                            if block['color'] == self.GRAY1:
                                block['color'] = self.GRAY2
                            else:
                                self.blocks.remove(block)
                            self.Ball_angle = 360 - self.Ball_angle
                            break
                

        # draw the black screen
        pygame.draw.rect(self.screen, BLACK, [10, 100, 780, 490])

        # draw the blocks
        for block in self.blocks[:]:
            pygame.draw.rect(self.screen, block['color'], block['block'])

        # draw the Player Base onto the surface
        pygame.draw.rect(self.screen, YELLOW, self.player)

        # draw the Ball onto the surface
        pygame.draw.circle(self.screen, RED, (int(self.Ball_x), int(self.Ball_y)), 10, 0)

        # check if numbers of blocks > 0
        if len(self.blocks) == 0:
            self.Ball_x = 400
            self.Ball_y = 560
            self.Ball_angle = random.randint(45, 135)
            self.moveLeft = False
            self.moveRight = False
            #self.niveis()
            self.level += 1
            if self.level == 7:
                self.level = 1
            self.next_exercise()

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        for event in events:
            if event.type is MOUSEBUTTONDOWN:
                pass
                # call your objects
            if event.type == KEYDOWN:
                # change the keyboard variables
                if event.key == K_LEFT:
                    self.moveRight = False
                    self.moveLeft = True
                if event.key == K_RIGHT:
                    self.moveLeft = False
                    self.moveRight = True
            if event.type == KEYUP:
                if event.key == K_LEFT:
                    self.moveLeft = False
                if event.key == K_RIGHT:
                    self.moveRight = False

        self.GameArea()

        pygame.display.flip()
        return 
        



# This module comes from the OLPC project and is licensed under the BSD license.
# The changes are minimal and only done to let it work outside Sugar.
# Added by Stas Zytkiewicz <stas.zytkiewicz@gmail.com> test code at the bottom
# of this module. be aware that the test code only works with childsplay installed
# and run within the childsplay_sp source directory.
# 

# Below is the original COPYING file.
##* Copyright (c) 2007, One Laptop Per Child.
##* All rights reserved.
##*
##* Redistribution and use in source and binary forms, with or without
##* modification, are permitted provided that the following conditions are met:
##*     * Redistributions of source code must retain the above copyright
##*       notice, this list of conditions and the following disclaimer.
##*     * Redistributions in binary form must reproduce the above copyright
##*       notice, this list of conditions and the following disclaimer in the
##*       documentation and/or other materials provided with the distribution.
##*     * Neither the name of One Laptop Per Child nor the
##*       names of its contributors may be used to endorse or promote products
##*       derived from this software without specific prior written permission.
##*
##* THIS SOFTWARE IS PROVIDED BY ONE LAPTOP PER CHILD ``AS IS'' AND ANY
##* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
##* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
##* DISCLAIMED. IN NO EVENT SHALL ONE LAPTOP PER CHILD BE LIABLE FOR ANY
##* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
##* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
##* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
##* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
##* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
##* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Implement Pygame's font interface using Pango for international support

Depends on:
    cairoimage (from the same source as this module)
    pygtk (to get the pango context)
    pycairo (for the pango rendering context)
    python-pango (obviously)
    numpy
    (pygame)

As soon as you import this module you have loaded *all* of the above.
You can still use pygame.font until you decide to call install(), which 
will replace pygame.font with this module.

Notes:

    * no ability to load TTF files, PangoFont uses the font files registered 
        with GTK/X to render graphics, it cannot load an arbitrary TTF file.  
        Most non-Sugar Pygame games use bundled TTF files, which means 
        that you will likely need at least some changes to your font handling.
        
        Note, however, that the Pygame Font class is available to load the TTF 
        files, so if you don't want to take advantage of PangoFont for already 
        written code, but want to use it for "system font" operations, you can 
        mix the two.
        
    * metrics are missing, Pango can provide the information, but the more 
        involved metrics system means that translating to the simplified model 
        in Pygame has as of yet not been accomplished.
        
    * better support for "exotic" languages and scripts (which is why we use it)

The main problem with SDL_ttf is that it doesn't handle internationalization 
nearly as well as Pango (in fact, pretty much nothing does). However, it is 
fairly fast and it has a rich interface. You should avoid fonts where possible, 
prerender using Pango for internationalizable text, and use Pango or SDL_ttf 
for text that really needs to be rerendered each frame. (Use SDL_ttf if profiling 
demonstrates that performance is poor with Pango.)

Note:
    Font -- is the original Pygame Font class, which allows you to load 
        fonts from TTF files/filenames
    PangoFont -- is the Pango-specific rendering engine which allows 
        for the more involved cross-lingual rendering operations.
"""
import pango
import logging
import pangocairo
import pygame.rect, pygame.image
import gtk
import struct
from pygame import surface
from pygame.font import Font
import cairoimage as _cairoimage

log = logging.getLogger('childsplay.pangofont')

# Install myself on top of pygame.font
def install():
    """Replace Pygame's font module with this module"""
    log.info( 'installing' )
    import pangofont
    import pygame
    pygame.font = pangofont
    import sys
    sys.modules["pygame.font"] = pangofont

class PangoFont(object):
    """Base class for a pygame.font.Font-like object drawn by Pango
    
    Attributes of note:
    
        fd -- instances Pango FontDescription object 
        WEIGHT_* -- parameters for use with set_weight
        STYLE_* -- parameters for use with set_style
        
    """
    WEIGHT_BOLD = pango.WEIGHT_BOLD
    WEIGHT_HEAVY = pango.WEIGHT_HEAVY
    WEIGHT_LIGHT = pango.WEIGHT_LIGHT
    WEIGHT_NORMAL = pango.WEIGHT_NORMAL
    WEIGHT_SEMIBOLD = pango.WEIGHT_SEMIBOLD
    WEIGHT_ULTRABOLD = pango.WEIGHT_ULTRABOLD
    WEIGHT_ULTRALIGHT = pango.WEIGHT_ULTRALIGHT
    STYLE_NORMAL = pango.STYLE_NORMAL
    STYLE_ITALIC = pango.STYLE_ITALIC
    STYLE_OBLIQUE = pango.STYLE_OBLIQUE
    def __init__(self, family=None, size=None, bold=False, italic=False, underline=False, fd=None):
        """If you know what pango.FontDescription (fd) you want, pass it in as
        'fd'.  Otherwise, specify any number of family, size, bold, or italic,
        and we will try to match something up for you."""
        # Always set the FontDescription (FIXME - only set it if the user wants
        # to change something?)
        if fd is None:
            fd = pango.FontDescription()
            if family is not None:
                fd.set_family(family)
            if size is not None:
                #log.debug( 'Pre-conversion size: %s', size )
                size = int(size*1024)
                #log.debug( 'Font size: %s', size, )
                fd.set_size(size) # XXX magic number, pango's scaling
        self.fd = fd
        self.set_bold( bold )
        self.set_italic( italic )
        self.set_underline( underline )

    def render(self, text, antialias=True, color=(255,255,255), background=None ):
        """Render the font onto a new Surface and return it.
        We ignore 'antialias' and use system settings.
        
        text -- (unicode) string with the text to render
        antialias -- attempt to antialias the text or not
        color -- three or four-tuple of 0-255 values specifying rendering 
            colour for the text 
        background -- three or four-tuple of 0-255 values specifying rendering 
            colour for the background, or None for trasparent background
        
        returns a pygame image instance
        """
        #log.debug( 'render: %r, antialias = %s, color=%s, background=%s', text, antialias, color, background )
        if not text:
            return surface.Surface((0, 0))
        layout = self._createLayout( text )
        # determine pixel size
        (logical, ink) = layout.get_pixel_extents()
        ink = pygame.rect.Rect(ink)

        # Create a new Cairo ImageSurface
        csrf,cctx = _cairoimage.newContext( ink.w, ink.h )
        cctx = pangocairo.CairoContext(cctx)

        # Mangle the colors on little-endian machines. The reason for this 
        # is that Cairo writes native-endian 32-bit ARGB values whereas
        # Pygame expects endian-independent values in whatever format. So we
        # tell our users not to expect transparency here (avoiding the A issue)
        # and we swizzle all the colors around.

        # render onto it
        if background is not None:
            background = _cairoimage.mangle_color( background )
            cctx.set_source_rgba(*background)
            cctx.paint()
        
        #log.debug( 'incoming color: %s', color )
        color = _cairoimage.mangle_color( color )
        #log.debug( '  translated color: %s', color )

        cctx.new_path()
        cctx.layout_path(layout)
        cctx.set_source_rgba(*color)
        cctx.fill()

        # Create and return a new Pygame Image derived from the Cairo Surface
        return _cairoimage.asImage( csrf )
    
    def set_bold( self, bold=True):
        """Set our font description's weight to "bold" or "normal"
        
        bold -- boolean, whether to set the value to "bold" weight or not 
        """
        if bold:
            self.set_weight(  self.WEIGHT_BOLD )
        else:
            self.set_weight(  self.WEIGHT_NORMAL )
    def set_weight( self, weight ):
        """Explicitly set our pango-style weight value"""
        self.fd.set_weight(  weight )
        return self.get_weight()
    def get_weight( self ):
        """Explicitly get our pango-style weight value"""
        return self.fd.get_weight()
    def get_bold( self ):
        """Return whether our font's weight is bold (or above)"""
        return self.fd.get_weight() >= pango.WEIGHT_BOLD
    
    def set_italic( self, italic=True ):
        """Set our "italic" value (style)"""
        if italic:
            self.set_style( self.STYLE_ITALIC )
        else:
            self.set_style( self.STYLE_NORMAL )
    def set_style( self, style ):
        """Set our font description's pango-style"""
        self.fd.set_style( style )
        return self.fd.get_style()
    def get_style( self ):
        """Get our font description's pango-style"""
        return self.fd.get_style()
    def get_italic( self ):
        """Return whether we are currently italicised"""
        return self.fd.get_style() == self.STYLE_ITALIC # what about oblique?
    
    def set_underline( self, underline=True ):
        """Set our current underlining properly"""
        self.underline = underline
    def get_underline( self ):
        """Retrieve our current underline setting"""
        return self.underline

    def _createLayout( self, text ):
        """Produces a Pango layout describing this text in this font"""
        # create layout
        layout = pango.Layout(gtk.gdk.pango_context_get())
        layout.set_font_description(self.fd)
        if self.underline:
            attrs = layout.get_attributes()
            if not attrs:
                attrs = pango.AttrList()
            attrs.insert(pango.AttrUnderline(pango.UNDERLINE_SINGLE, 0, 32767))
            layout.set_attributes( attrs )
        layout.set_text(text)
        return layout

    def size( self, text ):
        """Determine space required to render given text
        
        returns tuple of (width,height)
        """
        layout = self._createLayout( text )
        (logical, ink) = layout.get_pixel_extents()
        ink = pygame.rect.Rect(ink)
        return (ink.width,ink.height)
    
##    def get_linesize( self ):
##        """Determine inter-line spacing for the font"""
##        font = self.get_context().load_font( self.fd )
##        metrics = font.get_metrics()
##        return pango.PIXELS( metrics.get_ascent() )
##        def get_height( self ):
##        def get_ascent( self ):
##        def get_descent( self ):
        

class SysFont(PangoFont):
    """Construct a PangoFont from a font description (name), size in pixels,
    bold, and italic designation. Similar to SysFont from Pygame."""
    def __init__(self, name, size, bold=False, italic=False):
        fd = pango.FontDescription(name)
        fd.set_absolute_size(size*pango.SCALE)
        if bold:
            fd.set_weight(pango.WEIGHT_BOLD)
        if italic:
            fd.set_style(pango.STYLE_OBLIQUE)
        super(SysFont, self).__init__(fd=fd)

# originally defined a new class, no reason for that...
NotImplemented = NotImplementedError

def match_font(name,bold=False,italic=False):
    """Stub, does not work, use fontByDesc instead"""
    raise NotImplementedError("PangoFont doesn't support match_font directly, use SysFont or .fontByDesc")

def fontByDesc(desc="",bold=False,italic=False):
    """Constructs a FontDescription from the given string representation.
    
The format of the fontByDesc string representation is passed directly 
to the pango.FontDescription constructor and documented at:

    http://www.pygtk.org/docs/pygtk/class-pangofontdescription.html#constructor-pangofontdescription

Bold and italic are provided as a convenience.

The format of the string representation is:

  "[FAMILY-LIST] [STYLE-OPTIONS] [SIZE]"

where FAMILY-LIST is a comma separated list of families optionally terminated by a comma, STYLE_OPTIONS is a whitespace separated list of words where each WORD describes one of style, variant, weight, or stretch, and SIZE is an decimal number (size in points). For example the following are all valid string representations:

  "sans bold 12"
  "serif,monospace bold italic condensed 16"
  "normal 10"

The commonly available font families are: Normal, Sans, Serif and Monospace. The available styles are:
Normal  the font is upright.
Oblique the font is slanted, but in a roman style.
Italic  the font is slanted in an italic style.

The available weights are:
Ultra-Light the ultralight weight (= 200)
Light   the light weight (=300)
Normal  the default weight (= 400)
Bold    the bold weight (= 700)
Ultra-Bold  the ultra-bold weight (= 800)
Heavy   the heavy weight (= 900)

The available variants are:
Normal  
Small-Caps  

The available stretch styles are:
Ultra-Condensed the smallest width
Extra-Condensed 
Condensed   
Semi-Condensed  
Normal  the normal width
Semi-Expanded   
Expanded    
Extra-Expanded  
Ultra-Expanded  the widest width
    """
    fd = pango.FontDescription(name)
    if bold:
        fd.set_weight(pango.WEIGHT_BOLD)
    if italic:
        fd.set_style(pango.STYLE_OBLIQUE)
    return PangoFont(fd=fd)

def get_init():
    """Return boolean indicating whether we are initialised
    
    Always returns True 
    """
    return True

def init():
    """Initialise the module (null operation)"""
    pass

def quit():
    """De-initialise the module (null operation)"""
    pass

def get_default_font():
    """Return default-font specification to be passed to e.g. fontByDesc"""
    return "sans"

def get_fonts():
    """Return the set of all fonts available (currently just 3 generic types)"""
    return ["sans","serif","monospace"]


def stdcolor(color):
    """Produce a 4-element 0.0-1.0 color value from input"""
    def fixlen(color):
        if len(color) == 3:
            return tuple(color) + (255,)
        elif len(color) == 4:
            return color
        else:
            raise TypeError("What sort of color is this: %s" % (color,))
    return [_fixColorBase(x) for x in fixlen(color)]
def _fixColorBase( v ):
    """Return a properly clamped colour in floating-point space"""
    return max((0,min((v,255.0))))/255.0

if __name__ == '__main__':
    
    import SPLogging
    SPLogging.set_level('debug')
    SPLogging.start()
    log = logging.getLogger('childsplay.pangofont')
    
    # setup a test gettext environment
    import __builtin__ , os,gettext
    LOCALEDIR = '/usr/local/share/locale'
    if os.environ.has_key('LANGUAGE'):
        lang = os.environ['LANGUAGE']
        log.info("Setting locale to '%s'" % (lang))
        lang_trans = gettext.translation('childsplay',\
                                         localedir=LOCALEDIR,\
                                         languages=lang)
        __builtin__.__dict__['_'] = lang_trans.ugettext
    else:
        __builtin__.__dict__['_'] = lambda x:x
    
    from pygame.constants import *
    pygame.init()
    size = (600,200)
    scr = pygame.display.set_mode(size)
    fsize = 12
    y = 10
    fgcolor = (0,0,0)
    bgcolor = (200,200,200)
    scr.fill(bgcolor)
    pygame.display.update()

    text1 = "Childsplay and all the games are licensed with the GNU-GPL."
    text2 = "Classic memory game where you have to find pairs of sounds."
    for text in [text1,text2]:
        font = PangoFont(family='sans', size=fsize)
        font.set_bold(True)
        surf_org = font.render(text,True,fgcolor,None)# none means trasparent
        surf_ar = font.render(_(text),True,fgcolor,bgcolor)
    
        scr.blit(surf_org,(10,y))
        scr.blit(surf_ar,(10,y+30))
        y += 80
         
    clock = pygame.time.Clock()
    runloop = True
    while runloop :
        clock.tick(30)
        pygame.event.pump()
        events = pygame.event.get()
        for event in events:
            if event.type is KEYDOWN and event.key is K_ESCAPE or event.type is QUIT:
                runloop = False 
        pygame.display.update()
    
    
    



# -*- coding: utf-8 -*-

# Copyright (c) 2006 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           Timer.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import time

import atexit
import logging
import threading
from SPSpriteUtils import MySprite, SPSprite
from utils import char2surf

module_logger = logging.getLogger("childsplay.Timer")

# In the Timer related classes the logger is disabled because it can flood the
# logfile. (especially with find_char_sound)
class Worker(threading.Thread):
    """Thread object internally used and called by Timer.
    Don't use this directly, use Timer.""" 
    
    def __init__(self, delay=None, func=None, fargs=None, loop=1, lock=None, cb_func=None, now=False):
        self.logger = logging.getLogger("childsplay.Timer.Worker")
        threading.Thread.__init__(self)
        self.__delay = delay
        self.__func = func
        self.__fargs = fargs
        self.__loop = loop
        self.__do = 1
        self.__die = False
        self.__lock = lock
        self.__cb_func = cb_func
        self.__now = now
        atexit.register(self._stop)
        
    def __sleep(self, secs):
        """Sleep fuction that checks every tenth of a second to see if we should die"""
        while secs > 0:
            if not self.__do:
                break
            if self.__lock and self.__lock.locked():
                #print "locked continue to sleep"
                time.sleep(0.1)
                continue
            time.sleep(0.1)
            secs -= 0.1
    
    def run(self, now=False):
        """This overrides the threading.Thread.run method."""
        #self.logger.info("Timer started")
        if self.__now:
            apply(self.__func, self.__fargs)
        while self.__do:
            if self.__die:
                break
#            if self.__lock and self.__lock.locked():
#                print "locked and sleeping 100 msec"
#                time.sleep(0.1)
            self.__sleep(self.__delay)
#            if self.__lock and self.__lock.locked():
#                print "locked and again sleeping 100 msec"
#                time.sleep(0.1)
            try:
                if not self.__do:
                    # this is done because when we call _stop when we in time.sleep
                    # the __func will set it back and the thread will never die.
                    break
                self.__do = apply(self.__func, self.__fargs)
            except Exception:
                self.logger.exception("Timer stopped due to an exception in %s" % self.__func)
                self.__do = 0
            self.__loop -= 1
            if not self.__loop: self.__do = 0
        #self.logger.info("Timer thread stopped")
        if self.__cb_func:
            apply(self.__cb_func)
            
    def _stop(self):
        """Used to stop the thread.
        DON'T call this directly, use Timer.stop()."""
        self.__do = 0
        self.__die = True
            
class Timer:
    def __init__(self, delay=1, func=None, fargs=(), loop=-1, lock=None, cb_func=None, startnow=False):
        """Timer, a timer object which runs in a seperate thread and calls the
        function 'func' with the arguments 'fargs' after 'delay' seconds for 
        'loop' times.
        'fargs' defaults to (), and 'loop' defaults to -1 which means
        loop forever.
        lock is a threading.lock object
        cb_func is a function that is called when the timer stops. 
        The timer checks the return value of the called function and will stop
        when the return value is false.
        So you should make sure you return true when you want to continue and false
        on error or when it should stop.        
            
        After construction you must call the start method to actual start 
        the timer.
        The timer register a function in the atexit module to cleanup any threads
        still running when the main application exits.
        Be aware that if your application doesn't exit in a 'normal' way the 
        atexit function might not work. (not normal ways are exceptions that are
        not cached by your app and terminate the program.)        
        """
        self.logger = logging.getLogger("childsplay.Timer.Timer")
        #self.logger.debug("Timer created")
        self.worker = Worker(delay, func, fargs, loop, lock, cb_func, now=startnow)
    
    def start(self):
        """Start the timer object."""
        #self.logger.info("Starting timer...")
        self.worker.start()
                    
    def stop(self):
        """Stop the timer."""
        #self.logger.info("Stopping timer...")
        self.worker._stop()

class Clock(SPSprite):
    """"""
    def __init__(self, start='00:00', counting=1, end='10:00',
        clocksize=32, clockpos=(0, 0), clockcolor=(0, 0, 0),
        text='', textsize=18, textcolor=(0, 0, 0), textpos=1,
        alarm=None, lock=None):
        """Timer clock. This clock depends on SpriteUtils object which is part
        of the childsplay project (http://childsplay.sf.net).
        @start is the start value of the clock in a string and must contains 
        a colon as a seperation between minutes and seconds.
        @counting must be an integer 1 or -1 and is used to tell the clock to
        count up (1) or down (-1). You should make sure the @start has a suitable
        format to count up or down otherwise you could have negative values.
        @end is the end time in a string, format is the same as @start.
        @text is a string to be displayed together with the clock.
        @textpos can be 1 to 4 to set the position of the text related to the
        clock. 1 is on top, 2 right, 3 bottom, 4 left.
        @alarm is called when the end time is reached, should be a callable function.
        It defaults to None.
        The rest will be obvious.
        
        This class implements the Timer.Timer object so you should make sure
        you don't leave threads running in case of a exception."""
        self.logger = logging.getLogger("childsplay.Timer.Clock")
        self.__csurf = char2surf(start, clocksize, clockcolor)
        SPSprite.__init__(self, self.__csurf)
        
        self.__tsprite = MySprite(char2surf(text, textsize, textcolor))
        
        # convert textpos into a xy coordinate
        if textpos == 1:
            x = clockpos[0]
            y = clockpos[1] - textsize
        elif textpos == 3:
            x = clockpos[0]
            y = clockpos + clocksize
        elif textpos == 2:
            x = clockpos[0] + self.__csurf.get_width()
            y = clockpos[1] + \
                ((self.__csurf.get_height() / 2) - (self.__tsprite.get_sprite_height() / 2))
        elif textpos == 4:
            x = clockpos[0] - self.__tsprite.get_sprite_width() 
            y = clockpos[1] + \
                ((self.__csurf.get_height() / 2) - (self.__tsprite.get_sprite_height() / 2))
        self.__tpos = (x, y)
        m, s = start.split(':')
        self.__startminutes, self.__startseconds = int(m), int(s)
        self.__start = start
        m, s = end.split(':')
        self.__endminutes, self.__endseconds = int(m), int(s)
        self.__minutes = self.__startminutes
        self.__seconds = self.__startseconds
        self.__end = end
        self.__count = counting
        self.__csize = clocksize
        self.__cpos = clockpos
        self.__ccol = clockcolor
        self.__text = text
        self.__tsize = textsize
        self.__tcol = textcolor
        self.__alarm = alarm
        self.__clock_stopped = 0
        self.__clock = Timer(1, self.__update, loop=-1, lock=lock)
    
    def start_clock(self):
        self.display_sprite(pos=self.__cpos)
        self.__tsprite.display_sprite(pos=self.__tpos)
        self.__clock.start()
        
    def stop_clock(self):
        self.__clock.stop()
    def get_display(self):
        return (self.rect, self.image)
    def get_time(self):
        return "%02d:%02d" % (self.__minutes, self.__seconds)
        
    def __update(self):
        if self.__clock_stopped:
            return
        if self.__minutes == self.__endminutes and \
            self.__seconds == self.__endseconds:
            if self.__alarm:
                try:
                    apply(self.__alarm)
                except Exception:
                    self.logger.exception("Exception in %s" % self.__class__)
                self.__clock.stop()
                self.__clock_stopped = 1
                return
        if self.__count == 1:# forwards
            if self.__seconds == 59:
                self.__seconds = 0
                self.__minutes += self.__count
            else:
                self.__seconds += self.__count
        else:# backwards
            if self.__seconds == 0:
                self.__seconds = 59
                self.__minutes += self.__count
            else:
                self.__seconds += self.__count
        time = "%02d:%02d" % (self.__minutes, self.__seconds)
        self.__tsprite.erase_sprite()
        self.erase_sprite()
        self.image = char2surf(time, self.__csize, self.__ccol)
        self.display_sprite()
        self.__tsprite.display_sprite()
        return 1

if __name__ == '__main__':
    i = 0
    def test( *args):
        global i
        i += 1
        print i, "message from test"
        if args: 
            print "args from test", args
        return 1
        
    t = Timer(1, test, ('these are the args', ), loop=60)
    print "Start timer (60 times)"
    t.start()
    while i < 60:
        pass
    print "wait 2 seconds"
    time.sleep(2)
    print "and do some stuff in main"
    print "wait another 2 secs"
    time.sleep(2)
    print "and now stop the timer"
    t.stop()
    
